/***************************************************************************
                          casyncconn.cpp  -  description
                             -------------------
    begin                : Wed Jun 11 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "casyncconn.h"
#include "casyncsocketserver.h"
#include "cselector.h"
#include "cpoller.h"
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <netinet/in.h>        /* for sockaddr_in */
#include <sys/socket.h>        /* for AF_INET */
#include <netdb.h>             /* for gethostbyaddr */
#include <sys/time.h>          /* for gettimeofday */

#ifndef MSG_NOSIGNAL
#define MSG_NOSIGNAL 0
#endif

char *cAsyncConn::msBuffer = new char[MAX_MESS_SIZE+1];

cAsyncConn::cAsyncConn(int desc, cAsyncSocketServer *s, tConnType ct):
	cObj("cAsyncConn"),
	mSockDesc(desc),
	ok(desc>0),
	mWritable(true),
	mExtraPoll(0),
	mxServer(s),
	mAddrPort(0),
	mType(ct)
{
	struct sockaddr saddr;
	struct sockaddr_in *addr_in;
	struct hostent *hp;
	socklen_t addr_size = sizeof(saddr);
	unsigned int addr;
  
	mIp=0;
	ClearLine();
	mBufEnd = mBufReadPos = 0;
	mRegFlag = 0;
	if(mSockDesc)
	{
		if(0 > getpeername(mSockDesc, &saddr, &addr_size))
		{
			if(Log(2)) LogStream() << "Error getting peername, closing" << endl;
			CloseNow();
		}
		addr_in = (struct sockaddr_in *)&saddr;
		// copy IP
		addr = mIp = addr_in->sin_addr.s_addr;
		// asci ip addr
		mAddrIP = inet_ntoa(addr_in->sin_addr);
		// host name
		if( mxServer && mxServer->mUseDNS && (hp=gethostbyaddr((char *)&addr,sizeof(addr),AF_INET)))
			mAddrHost=hp->h_name;
		// port number
		mAddrPort=addr_in->sin_port;
	}
	memset (&mCloseAfter,0, sizeof(mCloseAfter));
}


/** connect to given host (ip) on port */
cAsyncConn::cAsyncConn(const string &host , int port):
	cObj("cAsyncConn"),
	mSockDesc(-1),
	ok(false),
	mWritable(true),
	mExtraPoll(0),
	mxServer(NULL),
	mAddrPort(port),
	mType(eCT_SERVER),
	mBufEnd(0),
	mBufReadPos(0),
	mRegFlag(0)
{
	ClearLine();
	Connect(host,port);
}

cAsyncConn::~cAsyncConn()
{
	close();
}

/** close connection to peer */
void cAsyncConn::close()
{
	if(mSockDesc <= 0) return;
	mWritable=false;
	ok=false;
	if(mxServer) mxServer->OnConnClose(this);
	TEMP_FAILURE_RETRY(::close(mSockDesc));
	mSockDesc=0;
}

/** flush as much from output buffer as possible to the iochannel */
void cAsyncConn::flush()
{
	int size=mBufSend.length();
	string empty("");
	if(size) Write(empty); // writes what is in the buffer already
}

/** register the connection into given selector */
int cAsyncConn::Register(cSelector& sel, int activity)
{
	if(mRegFlag == activity) return 0;
	mRegFlag = activity;
	sel.Register(mSockDesc, activity, this);
	return 1;
}

void cAsyncConn::UnRegister(cSelector &sel)
{
	if(!mRegFlag) return;
	if(sel.UnRegister(mSockDesc) <0)
	{
		if(ErrLog(1)) LogStream() << "Error in UnRegister" << endl;
		return;
	}
	mRegFlag = 0;
}

/** register the connection into given selector */
int cAsyncConn::Register(cPoller &pol , int activity)
{
	if(mRegFlag == activity) return 0;
	mRegFlag = activity;
	pol.Register(mSockDesc, activity, this);
	return 1;
}

/** register the connection into given selector */
void cAsyncConn::UnRegister(cPoller &pol )
{
	if(!mRegFlag) return;
	pol.UnRegister(mSockDesc);
	mRegFlag = 0;
}

/** reads a line from the msBuffer preaviously filled by ReadAll into private members */
int cAsyncConn::ReadLineLocal()
{
	if(!mxLine) throw "ReadLine with null line pointer";
	char *pos,*buf;
	buf = msBuffer + mBufReadPos;
	int len;
	len = mBufEnd - mBufReadPos;

	if(NULL == (pos = (char*)memchr(buf, mSeparator,len))) // 2do todo attention on ariving zero
	{
		if(mxLine->size()+len > mLineSizeMax)
		{
			CloseNow();
			return 0;
		}
		mxLine->append((char*)buf,len);
		mBufEnd = 0;
		mBufReadPos = 0;		
		return len;
	}
	len = pos - buf;

	mxLine->append((char*)buf,len);
	mBufReadPos += len+1;
	meLineStatus = AC_LS_LINE_DONE;
	
	return len+1;
}

/** you can provide a string for reading a line by this function
<precond>
LineStatus() == AC_LS_NO_LINE
strp != NULL
</precond> */
void cAsyncConn::SetLineToRead(string *strp,char delim, int max)
{
	if(LineStatus() != AC_LS_NO_LINE) throw "cAsyncConn::SetLineToRead - precondition not ok";
	if(!strp) throw "cAsyncConn::SetLineToRead - precondition not ok - null string pointer";
	meLineStatus = AC_LS_PARTLY;
	mLineSize = 0;
	mLineSizeMax = max;
	mxLine = strp;
	mSeparator = delim;
}

/** clears the line status */
void cAsyncConn::ClearLine()
{
	meLineStatus = AC_LS_NO_LINE;
	mLineSize = 0;
	mLineSizeMax = 0;
	mSeparator = '\n';
	mxLine = NULL;
}


/** return the line's pointer and keep it */
string * cAsyncConn::GetLine()
{
	return mxLine;
}

/** No descriptions */
void cAsyncConn::CloseNice(int msec)
{
	OnCloseNice();
	mWritable = false;
	if(msec <= 0) return CloseNow();
	if(!mBufSend.size()) return CloseNow();
	
	gettimeofday(&mCloseAfter,NULL);
	mCloseAfter.tv_usec += msec*1000;
	mCloseAfter.tv_sec  += mCloseAfter.tv_usec/1000000;
	mCloseAfter.tv_usec %= 1000000;
}

/** immediately close the connection */
void cAsyncConn::CloseNow()
{
	mWritable = false;
	ok=false;
//	if(mxServer) Register(mxServer->mSelector, SEL_CLOSE);
	if(mxServer) Register(mxServer->mPoller, POL_CLOSE);
}


/** reads all available data from the socket and stores it into a static member buffer */
int cAsyncConn::ReadAll(void)
{
	int buf_len , i=0;
	mBufReadPos = 0;
	mBufEnd = 0;
	if(!ok || !mWritable) return -1;

	while(
			((buf_len = recv(mSockDesc, msBuffer, MAX_MESS_SIZE, 0)) == -1) &&
			((errno == EAGAIN) || (errno == EINTR))
			&& (i++ <= 100)
		)::usleep(500);

	if(buf_len <= 0)
	{
		if(buf_len == 0)
		{
			/* Connection closed - hung up*/
			if(Log(2)) LogStream() << "User hung up.." << endl;
			CloseNow();
			return -1;
		}
		else
		{
			if(Log(2)) LogStream() << "Read IO Error: " << errno << " : " << strerror(errno) << endl;
			switch(errno)
			{
				case ECONNRESET:/* connection reset by peer */ break;
				case ETIMEDOUT:/* connection timed out */break;
				case EHOSTUNREACH:/* No route to host */ break;
				default:break;
			} // end switch
		}
		CloseNow();
		return -1;
	}
	else
	{
		// received data
		mBufEnd = buf_len;
		mBufReadPos = 0;
		msBuffer[mBufEnd]='\0'; // end the string
	}
	return buf_len;
}

/* Sends as many packets as it takes. */
/* This was taken from Beej's guide to network programming: */
/* http://www.ecst.csuchico.edu/~beej/guide/net/html/ */
int cAsyncConn::SendAll(const char *buf, size_t &len)
{
	size_t total = 0;        /* how many bytes we've sent */
	size_t bytesleft = len; /* how many we have left to send */
	int n = 0;
	int repetitions=0;

	while(total < len)
	{
		try
		{
			n = send(mSockDesc, buf + total, bytesleft, MSG_NOSIGNAL|MSG_DONTWAIT);
		}catch(...)
		{
			if(ErrLog(2))
				LogStream() << "exception in SendAll(buf," << len
					<< ") total=" << total
					<< " left=" << bytesleft
					<< " rep=" << repetitions
					<< " n=" << n << endl;
			return -1;
		}
		repetitions++;
		if(n == -1)
			break;
		total += n;
		bytesleft -= n;
	}

	len = total; /* return number actually sent here */
	return n == -1?-1:0; /* return -1 on failure, 0 on success */
}


/** connect to given host (ip) on port */
int cAsyncConn::Connect(const string &host, int port)
{
	struct sockaddr_in dest_addr;
	mSockDesc = CreateSock();

	if(mSockDesc == -1)
	{
		cout << "Error getting socket.\n" << endl;
		ok = false;
		return -1;
	}

	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(port);
	dest_addr.sin_addr.s_addr = inet_addr(host.data());
	memset(&(dest_addr.sin_zero), '\0', 8);

	int s = connect(mSockDesc, (struct sockaddr *)&dest_addr, sizeof(struct sockaddr));
	if(s==-1)
	{
		cout << "Error connectiong" << endl;
		ok=false;
		return -1;
	}
	ok=true;
	return 0;
}

/** create a socket */
int cAsyncConn::CreateSock()
{
	int sock, yes=1;
	/* Create socket */
	if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
		return -1;

	/* Fix the address already in use error */
	if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1)
		return -1;

	return sock;
}

/** binds given socket to a port */
int cAsyncConn::BindSocket(int sock, int port, const char *ia)
{
	if(sock <0) return -1;
	struct sockaddr_in my_addr;
	memset(&my_addr, 0, sizeof(struct sockaddr_in));
	my_addr.sin_family = AF_INET;
	my_addr.sin_addr.s_addr = INADDR_ANY; // default listen address
	if(ia) inet_aton(ia, &my_addr.sin_addr); // override it
	my_addr.sin_port = htons(port);

	/* Bind socket to port */
	if(bind(sock, (struct sockaddr *)&my_addr, sizeof(my_addr)) == -1)
	{
		return -1;
	}
	return sock;
}

/** listen on given socket */
int cAsyncConn::ListenSock(int sock)
{
	if(sock <0) return -1;
	if(listen(sock, 100) == -1)
	{
		return -1;
	}
	return sock;
}

/** set socket for non-blocking */
int cAsyncConn::NonBlockSock(int sock)
{
	if(sock<0) return -1;
	int flags;
	if((flags = fcntl(sock, F_GETFL, 0)) < 0)
	{
		return -1;
	}

	if(fcntl(sock, F_SETFL, flags | O_NONBLOCK) < 0)
	{
		return -1;
	}
	return sock;
}

/** No descriptions */
int cAsyncConn::ListenOnPort(int port, const char *ia)
{
	if(mSockDesc) return -1;

	mSockDesc=CreateSock();
	mSockDesc=BindSocket(mSockDesc,port,ia);
	mSockDesc=ListenSock(mSockDesc);
	mSockDesc=NonBlockSock(mSockDesc);
	ok = mSockDesc > 0;
	return mSockDesc;
}

/** return new socket */
int cAsyncConn::AcceptSock()
{
	int namelen;
	int socknum;
	int yes = 1;
	int i=0;
	struct sockaddr_in client;
	
	memset(&client, 0, sizeof(struct sockaddr_in));

	/* Get a socket for the connected user.  */
	namelen = sizeof(client);
	while(
		((socknum = accept(mSockDesc, (struct sockaddr *)&client, (socklen_t*)&namelen)) == -1) &&
		((errno == EAGAIN) || (errno == EINTR)) &&
		(i++ < 10))
	::usleep(50);

	if(socknum == -1) return -1;
	if(setsockopt(socknum, SOL_SOCKET, SO_KEEPALIVE, &yes, sizeof(int)) == -1)
	{
		::close(socknum);
		return -1;
	}
	if((socknum = NonBlockSock(socknum)) == -1) return -1;
	return socknum;
}

/** create new connection from the listening port */
cAsyncConn * cAsyncConn::Accept()
{
	int sd;
	cAsyncConn *new_conn;

	if((sd=AcceptSock()) < 0) return NULL;
	new_conn = mxServer->FactoryConnection(sd);
	if(!new_conn) throw "can't create connection";

	return new_conn;
}

/** Read property of tConnType mType. */
const tConnType& cAsyncConn::getType()
{
	return mType;
}

/** tests if ip matches a given mask or subnet */
int cAsyncConn::IPMatch(const char *ipmask)
{
	int bytes[4],mask;
	unsigned long ip;
	if((sscanf(ipmask, "%d.%d.%d.%d/%d",&bytes[0], &bytes[1], &bytes[2], &bytes[3], &mask) == 5)
		&& (mask > 0) && (mask <= 32))
	{
		ip = (bytes[0]<<24) | (bytes[1]<<16) | (bytes[2]<<8) | bytes[3];
		return (((0xFFFF << (32-mask)) & mIp) == ((0xFFFF << (32-mask)) & ip));
	}
	else
		return mAddrIP == ipmask;
}

/** tests if host matches a given mask or subnet */
int cAsyncConn::HostMatch(const char * ipmask)
{
	int len=strlen(ipmask);
	int pos=mAddrHost.length()-len;	if(pos < 0) pos=0;
	return 0==mAddrHost.compare(pos,len,ipmask);
}

/** this function is going to be executed periodicaly every N seconds by the  function of the same name in server */
int cAsyncConn::OnTimerBase()
{
	struct timeval now;
	gettimeofday(&now,NULL);

	if(
			mCloseAfter.tv_sec &&
			(
				(now.tv_sec > mCloseAfter.tv_sec) ||
				(
					now.tv_sec == mCloseAfter.tv_sec &&
					now.tv_usec >= mCloseAfter.tv_usec
				)
			)
		)
	{
		CloseNow();
		return 0;
	}

	OnTimer();
	return 0;
}

/** this is called every period of time */
int cAsyncConn::OnTimer()
{
	return 0;
}

/** this is called when write buffer gets empty */
void cAsyncConn::OnFlushDone()
{

}

/** write given data or store into buffer */
int cAsyncConn::Write(const string &data)
{
	static string tmp;

	// Append data to older data in buffer
	// but only if something is there and if free space
	// 1st) check the size
	if(mBufSend.size()+ data.size() >= MAX_SEND_SIZE)
	{
		if(Log(2)) LogStream() << "Buffer is too big, closing" << endl;
		CloseNow();
		return -1;
	}

	// a buffer from which to send data
	const char *send_buffer;
	// size of data to send
	size_t send_size;
	// indicator from which we rae sending
	bool appended;

	// second decide whether append or not
	if(mBufSend.size())
	{
		mBufSend.append(data);
		send_buffer= mBufSend.data();
		send_size  = mBufSend.size();
		appended   = true;
	}
	else
	{
		send_buffer= data.data();
		send_size  = data.size();
		appended   = false;
	}

	// if there isn't anything to send, don't bother
	if(!send_size) return 0;

	// make copy of send_size, size_sent will be modified by next function
	size_t size_sent = send_size;

	// now send the data, as much as possible
	if(SendAll(send_buffer,size_sent) == -1)
	{
		// analyze the error
		if((errno != EAGAIN) && (errno != EINTR))
		{
			// error during writing, remove the user
			if(Log(2)) LogStream() << "Error during writing, closing" << endl;
			CloseNow();
			return -1;
		}

		// if something has been sent, but not all, update the buffer
		if(size_sent)
		{
			// now it depends on if we appended or not
			// * if we appended, assign the rest of string to the buffer
			if(!appended)
				// this is supposed to actually reduce the size of mBufSend
				// it does a copy so it is slower but memory usage is important thing
				StrCutLeft(data, mBufSend, size_sent);
				//mBufSend = data.substr(size_sent,send_size-size_sent);
			// else cut off the beginning of the buffer
			else
				// this makes mBuffSend not grow
				//mBufSend = mBufSend.substr(size_sent, tmp.npos);
				StrCutLeft( mBufSend, size_sent);
				//tmp.assign(mBufSend,size_sent, tmp.npos);
				//mBufSend.reserve(0);
		}

		// Buffer overfill protection - only on registered connections
		if(mxServer && ok)
		{
			if(mBufSend.size() < MAX_SEND_UNBLOCK_SIZE )
			{
				//if(Register(mxServer->mSelector, SEL_INPUT|SEL_OUTPUT|SEL_DEFAULT))
				if(Register(mxServer->mPoller, POL_INPUT|POL_OUTPUT|POL_DEFAULT))
					if(Log(3)) LogStream() << "UnBlock INPUT or OUTPUT" << endl;
			}
			else if( mBufSend.size() >= MAX_SEND_FILL_SIZE )
			{
				//if(Register(mxServer->mSelector, SEL_OUTPUT|SEL_DEFAULT))
				if(Register(mxServer->mPoller, POL_OUTPUT|POL_DEFAULT))
					if(Log(3)) LogStream() << "Block INPUT" << endl;
			}
			mBufSend.reserve(unsigned(mBufSend.size() * 1.2));
		}
	}
	
	else // everything sent all right
	{
		// on success clear output buffer
		if(appended) mBufSend = mEmpty;

		// close nice was called, so we close now
		if(mCloseAfter.tv_sec) CloseNow();

		// unregister writing
		if(mxServer && ok)
		{
			//if(Register(mxServer->mSelector, SEL_INPUT|SEL_DEFAULT))
			if(Register(mxServer->mPoller, POL_INPUT|POL_DEFAULT))
				if(Log(3)) LogStream() << "UnBlock INPUT " << endl;
		}
		
		// report that is done
		OnFlushDone();
	}
	return size_sent;
}

/** function called before closing nicely */
int cAsyncConn::OnCloseNice(void)
{
	return 0;
}
