/***************************************************************************
                          casyncsocketserver.cpp  -  description
                             -------------------
    begin                : Tue Jun 10 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "casyncsocketserver.h"
#include <sys/socket.h>
#include <unistd.h>
#include <stdio.h>

#define DEF_POLL_SIZE 20480

/** create a server listening on the given port */
cAsyncSocketServer::cAsyncSocketServer(int port):
	cObj("cAsyncSocketServer"),
	mAddr("0.0.0.0"),
	timer_conn_period(4),
	timer_serv_period(2),
	mStepDelay(0),
	mMaxLineLength(10240),
	mUseDNS(0),
	mPoller(DEF_POLL_SIZE),
	mServerSock(0,this,eCT_LISTEN),
	mPort(port),
	mAccepting(false),
	mNowTreating(NULL)
{
}

cAsyncSocketServer::~cAsyncSocketServer()
{
	close();
}

/** runs the main loop while protected mbRun is true, that is set by stop() function or pause function */
void cAsyncSocketServer::run()
{
	mbRun = true;
	time_t now;
	if(Log(1)) LogStream() << "Main loop start." << endl;
	while(mbRun)
	{
//		if(Log(3)) LogStream() << "Next Step" << endl;
		try{
			//step();}
			TimeStep();}
		catch(const char *str){
			if(ErrLog(2)) LogStream() << "Exception: " << str << endl;}
		catch(...){
			throw "  cAsyncSocketServer::run()";}
		//onEveryStepInRun();

		now=::time(NULL);
		if(now >= mT.main + timer_serv_period)
		{
			mT.main = now;
			OnTimerBase();
		}
		if(mStepDelay) mSelector.sleep(mStepDelay);
	}
	if(Log(1)) LogStream() << "Main loop stop." << endl;
}

/** stop the main loop, do the last step and end it, doesn't close any connection */
void cAsyncSocketServer::stop()
{
	mbRun = false;
}

/** close the server with all connections, stop the loop and free all stuff */
void cAsyncSocketServer::close()
{
	mbRun = false;
	tCLIt it;
	for(it = mConnList.begin(); it != mConnList.end(); it++)
	{
		(*it)->close();
	}
	//try{mServerSock.shutdown(sockbuf::shut_readwrite);}catch(...){} // destructor does it itself
}

/** do one time step, accept incomming connection, take care of existing, close closed connections */
void cAsyncSocketServer::step()
{
	int i=0;

	try
	{
		int n = mSelector.poll(10);//poll();//0,0);
		if(!n)
		{
			mSelector.sleep(5000);
			return;
		}
	}
	catch(const char *str){cout << "Exception, after select: " << str << endl;return;}
	catch(...){throw "exception in step,select";}
	
	tSelectorKeyIterator it;
	for(it = mSelector.begin(); it != mSelector.end(); it ++)
	{
		mNowTreating = it->conn;
		if(!mNowTreating) continue;
		// some connections may have been disabled during this loop, so skip them
		if(it->conn->ok && it->activity & SEL_ACCEPT)
		{
			// accept incomming connection
			int i=0;
			cAsyncConn *new_conn;
			do
			{
				new_conn = accept();
				if(new_conn) addConnection(new_conn);
				i++;
			} while(new_conn && i <= 100);
		}
		
		if(it->conn->ok && it->activity & (SEL_INPUT))  // data to be read or data in buffer
		{
			// returns number of bytes read, on error returns negative code, when connection shell be closed
			try{if(input(it->conn) <= 0) it->conn->ok=false;}
			catch(const char *str){cout << "Exception in input: " << str << endl;}
			catch(...){throw "exception ass::select/input";}
		}
		if(it->conn->ok && it->activity & SEL_OUTPUT)
		{
			// NOTE: in sockbuf::write is a bug, missing buf increment, it will block until whole buffer is sent
			try{output(it->conn);}
			catch(const char *str){cout << "Exception in step 2: " << str << endl;}
			catch(...){throw "  cAsyncSocketServer::step 2()";}		if(++i>100)break; // next round
			
		}
		mNowTreating = NULL;
		if(!it->conn->ok || it->activity & (SEL_ERROR|SEL_CLOSE))
		{
			try{delConnection(it->conn);}
			catch(const char*str){cout << "Error in delcon: " << str << endl;}
			catch(...){throw "ass:step/delconn";}
		}
	}
}

/** listen and bind server to localhost address on port defined by SetListenPort function */
int cAsyncSocketServer::listen(int port)
{
	if(port) mPort=port;
	if(mServerSock.ListenOnPort(mPort,mAddr.data())<0) throw "Can't listen";
	//mServerSock.Register(mSelector,SEL_ACCEPT|SEL_DEFAULT);
	mServerSock.Register(mPoller,POL_ACCEPT|POL_DEFAULT);
	mAccepting = true;
	if(Log(0)) LogStream() << "Listening for connection on " << mAddr << ":" << mPort << endl;
	return 0;
}

/** Read property of int mPort. */
const int& cAsyncSocketServer::getmPort(){
	return mPort;
}

/** Write property of int mPort. */
void cAsyncSocketServer::setmPort( const int& _newVal){
	mPort = _newVal;
}

/** create a connection object to the sock descriptor */
cAsyncConn * cAsyncSocketServer::FactoryConnection(int sd)
{
	return new cAsyncConn(sd, this);
}

/** accept connection and return  connection pointer or NULL if error */
cAsyncConn * cAsyncSocketServer::accept()
{
	if(!AllowNewConn()) return NULL;
	return mServerSock.Accept();
}

/** add given connection to server, the pointer is managed then by the server,
	it's deleted, when no more use for it*/
void cAsyncSocketServer::addConnection(cAsyncConn *new_conn)
{
	if(!new_conn) throw "addConnection null pointer";
	if(!new_conn->ok)
	{
		if(new_conn->Log(3)) new_conn->LogStream() << "Access refused " << new_conn->AddrIP() << endl;
		delete new_conn;
		return;
	}
//	new_conn->Register(mSelector, SEL_INPUT|SEL_DEFAULT);
	new_conn->Register(mPoller, POL_INPUT|POL_DEFAULT);
	mConnList.push_back(new_conn);
	if(0 > OnNewConn(new_conn)) delConnection(new_conn);
}

/** remove given connection from server */
void cAsyncSocketServer::delConnection(cAsyncConn *old_conn)
{
	if(!old_conn) throw "delConnection null pointer";
	if(mNowTreating == old_conn)
	{
		old_conn->ok = false;
		return;
	}
	tCLIt it=find(mConnList.begin(), mConnList.end(), old_conn);
	cAsyncConn *found=(*it);
	if( (it ==  mConnList.end()) || (found != old_conn) )
	{
		cout << "not found " << old_conn << endl;
		throw "Trying to delete non-existent connection";
	}
	mConnList.erase(it);
	//cout << "Deleting connection" << old_conn << endl;
	delete old_conn;
}

/** perform input operation, read all data from the connection, return number of bytes read, return negative number, if error occured */
int cAsyncSocketServer::input(cAsyncConn *conn)
{
	int just_read=0;
	// read all data available iinto a buffer
	try{
		if(conn->ReadAll()<=0) return 0;
	}catch(...){cout << "Exception in ReadAll" << endl; return 0;}
	while(conn->ok && conn->mWritable)
	{
		// create new line obj if necessary
		if(conn->LineStatus() == AC_LS_NO_LINE)
			conn->SetLineToRead(FactoryString(conn),'|',mMaxLineLength);
		// read data into it from the buffer
		just_read += conn->ReadLineLocal();
		/// test line done
		if(conn->LineStatus() == AC_LS_LINE_DONE)
		{
			OnNewMessage(conn,conn->GetLine());
			conn->ClearLine();
			// connection may be closed after this
		}
		if( conn->BufferEmpty() ) break;
	}
	return just_read;
}

/** perform output operation */
int cAsyncSocketServer::output(cAsyncConn * conn)
{
	conn->flush();
	return 0;
}

/** treat message for given connection */
void cAsyncSocketServer::OnNewMessage(cAsyncConn *, string *str)
{
	
	delete str;
}

/** create somehow a string to get line for given connection, ad return th pointer */
string * cAsyncSocketServer::FactoryString(cAsyncConn *)
{
	return new string;
}

/** trigger is called when connection is closed */
void cAsyncSocketServer::OnConnClose(cAsyncConn* conn)
{
	if(!conn) return;
	conn->UnRegister(mPoller);
}

/** return negative if conn should be removed or error */
int cAsyncSocketServer::OnNewConn(cAsyncConn*conn)
{
	if(!conn) return -1;
	return 0;
}

/** this function is going to be executed periodicaly every N seconds and will call function of the same name in evey connection */
int cAsyncSocketServer::OnTimerBase()
{
	tCLIt it;
	time_t now=time(NULL);
	OnTimer();
	if( (mT.conn + timer_conn_period) <= now)
	{
		mT.conn = now;
		for(it=mConnList.begin(); it != mConnList.end(); it++)
			if((*it)->ok) (*it)->OnTimerBase();
	}
	return 0;
}

/** this is called every period of time */
int cAsyncSocketServer::OnTimer()
{
	return 0;
}

/** do one time step, accept incomming connection, take care of existing, close closed connections */
void cAsyncSocketServer::TimeStep()
{
	int i=0;
	try
	{
		int n = mPoller.poll(10);//poll();//0,0);
		if(!n)
		{
			mSelector.sleep(5000);
			return;
		}
	}
	catch(const char *str){cout << "Exception, after poll: " << str << endl;return;}
	catch(...){throw "exception in step,poll";}

	cPoller::iterator it;
	for(it = mPoller.begin(); it != mPoller.end(); ++it)
	{
		mNowTreating = mPoller.Conn(it);
		cAsyncConn *conn = mNowTreating;
		int activity = (mPoller.Actions(it) & POL_STANDARD) + conn->mExtraPoll;
		bool &OK = conn->ok;
		
		if(!mNowTreating) continue;
		// some connections may have been disabled during this loop, so skip them
		if(OK && activity & POL_ACCEPT && conn->GetType() == eCT_LISTEN)
		{
			// accept incomming connection
			int i=0;
			cAsyncConn *new_conn;
			do
			{
				new_conn = accept();
				if(new_conn) addConnection(new_conn);
				i++;
			} while(new_conn && i <= 50);
		}

		if(OK && activity & POL_INPUT  && conn->GetType() == eCT_CLIENT)  // data to be read or data in buffer
		{
			// returns number of bytes read, on error returns negative code, when connection shell be closed
			try{if(input(conn) <= 0) OK=false;}
			catch(const char *str){cout << "Exception in input: " << str << endl;}
			catch(...){throw "exception ass::select/input";}
		}
		if(OK && activity & POL_OUTPUT)
		{
			// NOTE: in sockbuf::write is a bug, missing buf increment, it will block until whole buffer is sent
			try{output(conn);}
			catch(const char *str){cout << "Exception in step 2: " << str << endl;}
			catch(...){throw "  cAsyncSocketServer::step 2()";}		if(++i>100)break; // next round

		}
		mNowTreating = NULL;
		if(!OK || activity & (POL_ERROR|POL_CLOSE))
		{
			try{delConnection(conn);}
			catch(const char*str){cout << "Error in delcon: " << str << endl;}
			catch(...){throw "ass:step/delconn";}
		}
	}
}
