/***************************************************************************
                          cconfigbase.h  -  description
                             -------------------
    begin                : Sun Jun 15 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CCONFIGBASE_H
#define CCONFIGBASE_H

#include "tstringhashmap.h"
#include "cconfigitembase.h"
#include "cobj.h"

namespace nConfig
{

/**configuration base class
  *@author Daniel Muller
  */

class cConfigBase : public cObj
{
public:
	cConfigBase();
	virtual ~cConfigBase();
	/** save config, to be able to load it after */
	virtual int Save() = 0;
	/** The config load function - whetre from a file, or from database, or whatever
		return >= 0 on success, otherwise error code
	*/
	virtual int Load() = 0;

	/********** typedefs */
	/** the itemlist type */
	// nicked acess
	typedef tStringHashMap(cConfigItemBase*) tItemHash;
	typedef pair<string,cConfigItemBase*> tNickItemPair;
	typedef tItemHash::iterator tIHIt;
	// numerated access
	typedef vector<cConfigItemBase*> tItemVec;
	typedef tItemVec::iterator tIVIt;

	// set the base pointer to relative adresses
	void SetBaseTo(void * new_base);

	void *mBasePtr;
	tItemHash mhItems;
	tItemVec  mvItems;

	/** add existing item pointed by the argument at the end of mvItems */
	cConfigItemBase * Add(cConfigItemBase *);

	/** add existing item pointed by the argument at the end of mvItems , and bind a nick to it*/
	cConfigItemBase * Add(const string &, cConfigItemBase *);

	/** bind a nick to a given vaiable */
	void BindNick(int , const string &);

	/** create and add an item of template type with nick too */
	template <class TYPE> cConfigItemBase * Add(const string &, TYPE &);
	template <class TYPE> cConfigItemBase * Add(const string &, TYPE &, const TYPE &def);

	/** access operators */
	cConfigItemBase * operator[](int);
	cConfigItemBase * operator[](const string &);
	const cConfigItemBase * operator[](int) const;
};

/** create and add an item of template type with nick too */
template <class TYPE> cConfigItemBase * cConfigBase::Add(const string &name, TYPE &var)
{
	cConfigItemBase * ci = new cConfigItemBaseT<TYPE>(var);
	return Add(name, ci);
}

template <class TYPE> cConfigItemBase * cConfigBase::Add(const string &name, TYPE &var, const TYPE &def)
{
	cConfigItemBase * ci = Add(name, var);
	*ci=def;
	return ci;
}

};
#endif
