/***************************************************************************
 *   Copyright (C) 2003 by Dan Muller                                      *
 *   dan@verliba.cz                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef CCONFIGITEMBASE_H
#define CCONFIGITEMBASE_H

#include <string>
#include <iostream>

using std::string;
using std::istream;
using std::ostream;

namespace nConfig
{

typedef enum
{
	eIT_VOID, // unspecified
	eIT_NTSTR, // null terminated string
	eIT_INT,
	eIT_UINT, // unsigned int
	eIT_LONG,
	eIT_ULONG,
	eIT_LLONG, // long long
	eIT_ULLONG,
	eIT_AR_BYTE, // array of bytes
	eIT_STRING  // std::string
}tItemType;


/**
a base class for all kinds of config items, used by cConfigBase

@author Daniel Muller
*/
class cConfigItemBase
{
	friend class mConfigBase;
public:
	/** constructors */
	cConfigItemBase(void *addr=0, void *base=0) :
		mAddr(addr),
		mBase(base)
	{};

/*	template <class TYPE> cConfigItemBase & Bind(TYPE *addr, int relative=0)
	{
		mAddr = (void*) addr;
		return *this;
	};*/

	/** setting value */
	template <class TYPE> cConfigItemBase &operator=(const TYPE &i){*(TYPE*)Address() = i; return *this;};
	/** cast operator */
	template <class TYPE> operator TYPE(){return *(TYPE*)mAddr;}

	/** input /output */
	virtual istream &ReadFromStream(istream &)=0;
	virtual ostream &WriteToStream (ostream &)=0;
	friend istream &operator >> (istream &is, cConfigItemBase &ci){return ci.ReadFromStream(is);}
	friend ostream &operator << (ostream &os, cConfigItemBase &ci){return ci.WriteToStream (os);}

	/** conversion of a string to a value */
	virtual void ConvertFrom(const std::string &) = 0;
	void *mAddr;

protected:
	virtual void *Address(){return mAddr;}
	void *mBase;
};

template <class TYPE> class cConfigItemBaseT : public cConfigItemBase
{
public:
	// constructor
	cConfigItemBaseT(TYPE &var): cConfigItemBase((void*)&var, 0){}

	TYPE & Data(){return *(TYPE*)mAddr;}
	// methods
	virtual istream &ReadFromStream(istream& is){ return is >> this->Data(); }
	virtual ostream &WriteToStream (ostream& os){ return os << this->Data(); }
	virtual TYPE *Address(){return (TYPE*)cConfigItemBase::Address();}
	virtual void ConvertFrom(const std::string &str);
	cConfigItemBaseT<TYPE> &operator=(const TYPE &i){*(TYPE*)Address() = i; return *this;};
	//cConfigItemBaseT<TYPE> &operator=(const TYPE i ){*(TYPE*)Address() = i; return *this;};
};

};
#endif
