/***************************************************************************
                          cconndc.cpp  -  description
                             -------------------
    begin                : Fri May 9 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cconndc.h"
#include <sys/time.h>     /* for gettimeofday */

namespace nDirectConnect
{

cConnDC::cConnDC(int sd, cAsyncSocketServer *server)
	: cAsyncConn(sd,server)
{
//	mpMsg = NULL;
	mpUser = NULL;
	SetClassName("ConnDC");
	mLogStatus = 0;
	memset(&mTO,0, sizeof(mTO));
	mBan.mBanType = 0;
	mFeatures = 0;
	mNIHubs=-1;
	mNISlots=-1;
	mNIActive=-1;
	mIPValid=eVI_OK;
	mSendNickList = false;
//	SetTimeOut(eTO_LOGIN,180); // timeout to login on 20s
}

cConnDC::~cConnDC()
{
//	if(mpMsg) delete mpMsg;
	if(GetLSFlag(eLS_ALOWED) && mxServer)
	{
		Server()->mUserCount --;
		if(mpUser) Server()->mTotalShare -= mpUser->mShare;
	}
	if(mpUser) delete mpUser;
}

/** returns true if ok, unless false */
bool cConnDC::SetUser(cUser *usr)
{
	if(!usr)
	{
		if(ErrLog(0)) LogStream() << "Trying to add a NULL user" << endl;
		return false;
	}
	if(mpUser)
	{
		if(ErrLog(1)) LogStream() << "Trying to add user when it's actually done" << endl;
		delete usr;
		return false;
	}
	mpUser = usr;
	mpUser->mxConn = this;
	mpUser->mxServer = (cServerDC *) mxServer;
	if(Log(3)) LogStream() << "User " << usr->mNick << " connected ... " << endl;	
	return true;
}

/** Send raw data whenever it's next possible,
		n is the size of data, if not specified, or zero, the null terminated string is mesured */
int cConnDC::Send(string & data)
{
	if(!mWritable) return 0;
	if(data.size() >= MAX_SEND_SIZE-1)
	{
		if(Log(2))
			LogStream() << "Truncating too long message from: "
				<< data.size() << " to " << MAX_SEND_SIZE -1 << " Message starts with: " << data.substr(0,10) << endl;
		data.resize( MAX_SEND_SIZE -1,' ');
	}
	if(data[data.size()-1]!='|') data.append("|");
	if(Log(5)) LogStream() << "OUT: " << data.substr(0,100) << endl;
	if(msLogLevel >= 3)
		Server()->mNetOutLog << data.size() << " "
			<< data.size() << " "
			<< 1 << " " << data.substr(0,10) << endl;

	return Write(data);
}

/** log the event */
int cConnDC::StrLog(ostream & ostr, int level)
{
	if(cObj::StrLog(ostr,level))
	{
		LogStream()   << "(" << mAddrIP ;//<< ":" << mAddrPort;
		if(mAddrHost.length())
			LogStream() << " " << mAddrHost;
		LogStream()   << ") ";
		if(mpUser)
			LogStream() << "[ " << mpUser->mNick << " ] ";
		return 1;
	}
	return 0;
}

/** set log status flag to a given one in the user*/
void cConnDC::SetLSFlag(unsigned int st)
{
	mLogStatus |= st;
}

/** get log status flag to a given one in the user */
unsigned int cConnDC::GetLSFlag(unsigned int st)
{
	return mLogStatus & st;
}

/** create somehow a string to get line for given connection, ad return th pointer */
string * cConnDC::FactoryString()
{
//	mpMsg = new cMessageDC();
//	return &(mpMsg->GetStr());
	mMsg.ReInit();
	return &(mMsg.GetStr());
}

/** this is called every period of time */
int cConnDC::OnTimer()
{
	ostringstream os;
	string omsg;
	// check the timeouts
	int i;
	for(i=0; i < eTO_MAXTO; i++)
	{
		if(!CheckTimeOut(tTimeOut(i)))
		{
			os << "Operation timeout (" << tTimeOut(i) << ")";
			if(Log(2)) LogStream() << "Operation timeout (" << tTimeOut(i) << ")" << endl;
			Server()->ConnCloseMsg(this,os.str(),6000);
			break;
		}
	}

	// check frozen users, send to every user, every minute an empty message
	if(
		Server()->MinDelay(mT.ping,60) &&
		mpUser &&
		mpUser->mInList &&
		mpUser->mT.login < ( time(NULL) - 600 )
		)
	{
		omsg="";
		Send(omsg);
	}

	// upload line optimisation  - upload userlist slowlier
	if(mpUser && mpUser->mQueueUL.size())
	{
		unsigned pos,ppos=0;
		string buf,nick;
		cUser *other;
		for(i=0; i < Server()->mC.ul_portion; i++)
		{
			pos=mpUser->mQueueUL.find_first_of('|',ppos);
			if(pos == mpUser->mQueueUL.npos) break;

			nick = mpUser->mQueueUL.substr(ppos, pos-ppos);
			other = Server()->FindUserNick ( nick );
			ppos=pos+1;

			// check if user found
			if(!other)
			{
				if(nick != Server()->mC.hub_security)
				{
					buf.append("$Quit ");
					buf.append(nick);
				}
			}
			else
			{
				buf.append("$MyINFO $ALL ");
				buf.append(other->mNick);
				buf.append(" ");
				buf.append(other->mMyINFO);
			}
			buf.append("|");
		}
		Send(buf);

		if(pos != mpUser->mQueueUL.npos) pos++;
		// I can spare some RAM here by copying it to intermediate buffer and back
		mpUser->mQueueUL.erase(0,pos);
		mpUser->mQueueUL.reserve(0);
	}
	
	return 0;
}

/** Set's the timeout for given action, after timeout if not cleared, connection is closed */
int cConnDC::SetTimeOut(tTimeOut t, long sec, long usec)
{
	if(t >= eTO_MAXTO) return 0;
	gettimeofday(&mTO[t],NULL);
	mTO[t].tv_sec  += sec;
	mTO[t].tv_usec += usec;
	mTO[t].tv_sec  += mTO[t].tv_usec/1000000;
	mTO[t].tv_usec %= 1000000;

	return 1; //ok
}

/** storno the timeout */
int cConnDC::ClearTimeOut(tTimeOut t)
{
	if(t >= eTO_MAXTO) return 0;
	memset(&mTO[t],0,sizeof(timeval));
	return 1; //ok
}

/** return true if timeout is set */
int cConnDC::IsSetTimeOut(tTimeOut t)
{
	if(mTO[t].tv_sec) return 1;
	return 0;
}

/** return true if time is not out yet */
int cConnDC::CheckTimeOut(tTimeOut t)
{
	if(t >= eTO_MAXTO) return 0;
	if(!IsSetTimeOut(t)) return 1;
	struct timeval now;
	gettimeofday(&now,NULL);
	if(now.tv_sec < mTO[t].tv_sec) return 1;
	if(now.tv_sec > mTO[t].tv_sec) return 0;
	if(now.tv_usec > mTO[t].tv_usec) return 0;
	return 1;
}

/** this is called when write buffer gets empty */
void cConnDC::OnFlushDone()
{
	mBufSend.reserve(0);
	// if user is at least 10 minutes in user list, set timeout on next flush on 5 minutes
	// this works only if something happens on the hub, 
	// so there is in timer, to send every minute en empty message
////	if(
////		mpUser &&
////		mpUser->mInList &&
////		mpUser->mT.login < ( time(NULL) - 600 )
////		)
////		SetTimeOut(eTO_FLUSH,300);
//	if(IsSetTimeOut(eTO_FLUSH))
//		ClearTimeOut(eTO_FLUSH);
}

/** function called before closing nicely */
int cConnDC::OnCloseNice()
{
	if(mxServer)
	{
		string omsg="$ForceMove ";
		int i= (int) (10.0*rand()/(RAND_MAX+1.0));
		omsg+=Server()->mC.redir_host[i];
		Send(omsg);
	}
	return 0;
}
};
