/***************************************************************************
                          cconndc.h  -  description
                             -------------------
    begin                : Fri May 9 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CCONNDC_H
#define CCONNDC_H

#include "casyncconn.h"
#include "cserverdc.h"
#include "cmessagedc.h"
#include "cuser.h"
#include "creguserinfo.h"
#include "cdcbanrecord.h"

namespace nDirectConnect
{

class cUser;
class cServerDC;


/** LOGIN STATUS, unless all these aren't ok, user won't be let to do no more
 * than something .. */

typedef enum
{
	eLS_KEYOK   = 1 << 0, // the key was all right
	eLS_VALNICK = 1 << 1, // validate valid nick
	eLS_PASSWD  = 1 << 2, // password was ok, if userneeds no passw this is set to true
	eLS_VERSION = 1 << 3, // version of client is ok
	eLS_MYINFO  = 1 << 4, // myinfo is received and ok
	eLS_ALOWED  = 1 << 5, // user is allowd to enter the hub, hub isn't full
	eLS_LOGIN_DONE = eLS_KEYOK|eLS_VALNICK|eLS_PASSWD|eLS_VERSION|eLS_MYINFO|eLS_ALOWED // all
} tLogStatus;

typedef enum
{
	eTO_KEY=0,  // after lock, waiting for key
	eTO_LOGIN,  // after accept, until DoUserLogin
	eTO_MYINFO, // after getionfo until myinfo
	eTO_FLUSH,  // from some wites to flush
	eTO_MAXTO   // no use
} tTimeOut;

// forward declarations
class cDCBanRecord;

/**a direct connect client connection
  *@author Daniel Muller
  */


//***************** the class
class cConnDC : public cAsyncConn
{
	friend class cDCProto;
public:
	cConnDC(int sd=0, cAsyncSocketServer *server=NULL);
	virtual ~cConnDC();
	/** returns true if ok, unless false */
	bool SetUser(cUser *usr);
	/** set log status flag to a given one in the user*/
	void SetLSFlag(unsigned int st);
public: // Public attributes
	/** a pointer to a currently treated DC message */
	//cMessageDC * mpMsg;
	cMessageDC mMsg;
	/** a pointer to dc user, this is the only class that can destroy it,
		and that must keep it until it's end, server creates it */
	cUser * mpUser;
	/** client's version string */
	string mVersion;
	/** user's ban record */
	cDCBanRecord mBan;
	/** supported extended protocol features */
	unsigned mFeatures;
	/** connected hubs */
	int mNIHubs;
	/** opened slots*/
	int mNISlots;
	/** active or passive (0 passive, 1 active, else unknown)*/
	int mNIActive;
	/** the validity of connection's ip */
	int mIPValid;
	/** info about registration */
	cRegUserInfo mRegInfo;
	/** if nicklist is to send on userlogin */
	bool mSendNickList;
	/** return pointer to the server */
	inline cServerDC * Server(){return (cServerDC*) mxServer;}
	/** Send raw data whenever it's next possible,
	n is the size of data, if not specified, or zero, the null unterminated string is mesured
	the dc separator is appended |
	*/
	//int Send(const char * data, int n=-1);
	/** Send raw data whenever it's next possible,
		n is the size of data, if not specified, or zero, the null terminated string is mesured */
	int Send(string & data);
	/** log the event */
	virtual int StrLog(ostream & ostr, int level);
	/** get log status flag to a given one in the user */
	unsigned int GetLSFlag(unsigned int st);
	/** create somehow a string to get line for given connection, ad return th pointer */
	string * FactoryString();
	/** this is called every period of time */
	int OnTimer();
private:
	/** login status */
	unsigned int mLogStatus;

public: // Protected methods
	/** Set's the timeout for given action, after timeout if not cleared, connection is closed */
	int SetTimeOut(tTimeOut, long sec=10, long usec=0);
	/** storno the timeout */
	int ClearTimeOut(tTimeOut);
	/** return true if time is not out yet */
	int CheckTimeOut(tTimeOut t);
	/** this is called when write buffer gets empty */
	void OnFlushDone();
	/** return true if timeout is set */
	int IsSetTimeOut(tTimeOut);
protected: // Protected attributes
	/** time out's */
	struct timeval mTO[eTO_MAXTO];
	/** time ins */
	struct sTimes
	{
		time_t key;
		time_t ping;
		sTimes():key(0),ping(0){};
	};
	sTimes mT;
protected: // Protected methods
	/** function called before closing nicely */
	int OnCloseNice();
};

};

#endif
