/***************************************************************************
                          cdcconsole.cpp  -  description
                             -------------------
    begin                : Wed Jul 2 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cdcconsole.h"
#include "cserverdc.h"
#include "cuser.h"
#include "ckick.h"
#include "cinterpolexp.h"
#include <sstream>
#include <iomanip>

namespace nDirectConnect
{

cDCConsole::cDCConsole(cServerDC &s):
	cObj("nDC::Concole"),
	mS(s),
	mTriggers(mS.mMySQL)
{
	
}
cDCConsole::~cDCConsole(){
}

/** act on op's command */
int cDCConsole::OpCommand(const string &str, cConnDC * conn)
{
	stringstream cmd_line(str);
	string cmd;
	cmd_line >> cmd;
	
	if(!conn || !conn->mpUser) return 0;
	tUserCl cl=conn->mpUser->mClass;

	switch(cl)
	{
		case eUC_MASTER:
			if( cmd == "!quit"      ) return CmdQuit(cmd_line,conn);
			if( cmd == "!hublist"   ) { mS.RegisterInHublist(mS.mC.hublist_host, mS.mC.hublist_port); return 1;}
		case eUC_ADMIN:
			if( cmd == "!set"       || cmd=="!="  ) return CmdSet(cmd_line,conn);
			if( cmd == "!regpurge"  ) return CmdRegPurge( cmd_line, conn);
			if( cmd == "!getconfig" || cmd=="!gc" ) return CmdGetconfig(cmd_line,conn);
			if( cmd == "!userlimit" || cmd=="!ul" ) return CmdUserLimit(cmd_line,conn);
			if( cmd == "!reload"    || cmd=="!re" ) return CmdReload(cmd_line,conn);
		case eUC_CHEEF:
			if( cmd == "!broadcast" || cmd=="!bc" ) return CmdBroadcast(cmd_line,conn,eUC_NORMUSER,eUC_MASTER);
			if( cmd == "!regdeluser"|| cmd=="!rd" ) return CmdRegDelUser( cmd_line, conn);
			if( cmd == "!regnewuser"|| cmd=="!rn" ) return CmdRegNewUser( cmd_line, conn);
			if( cmd == "!regclass"  || cmd=="!rc" ) return CmdRegClass  ( cmd_line, conn);
			if( cmd == "!hostban"   || cmd=="!bh" ) return CmdBan(cmd_line,conn,true,eBT_HOST);
			if( cmd == "!hostunban" || cmd=="!uh" ) return CmdUnban(cmd_line, conn, eBT_HOST);
			if( cmd == "!drop"      || cmd=="!dr" ) return CmdKick(cmd_line,conn, true);
			if( cmd == "!whoip"     || cmd=="!wi" ) return CmdWhoIP(cmd_line, conn);
			if( cmd == "!class"                   ) return CmdClass     (cmd_line, conn);
			if( cmd == "!protect"                 ) return CmdProtect   (cmd_line, conn);
		case eUC_OPERATOR:
			if( cmd == "!nickban"    || cmd=="!bn" ) return CmdBan(cmd_line,conn,true , eBT_NICK);
			if( cmd == "!ban"        || cmd=="!bi" ) return CmdBan(cmd_line,conn, true, eBT_NICKIP);
			if( cmd == "!tempban"    || cmd=="!tb" ) return CmdBan(cmd_line,conn,false, eBT_NICKIP);
			if( cmd == "!ipban"      || cmd=="!bi" ) return CmdBan(cmd_line,conn, true, eBT_IP);
			if( cmd == "!nickunban"  || cmd=="!un" ) return CmdUnban(cmd_line, conn, eBT_NICK);
			if( cmd == "!unban"      || cmd=="!ub" ) return CmdUnban(cmd_line, conn, eBT_NICKIP);
			if( cmd == "!ipunban"    || cmd=="!ui" ) return CmdUnban(cmd_line, conn, eBT_IP);
			if( cmd == "!ipbanlist"  || cmd=="!bl" ) return CmdBanList(cmd_line, conn, eBT_IP);
			if( cmd == "!ipbaninfo"  || cmd=="!ibi" ) return CmdBanList(cmd_line, conn, eBT_IP, true);
//			if( cmd == "!nickbanlist"|| cmd=="!nl" )
//			if( cmd == "!reglist"    || cmd=="!rl" )
			if( cmd == "!kick"       || cmd=="!ki" ) return CmdKick(cmd_line,conn, false);
			if( cmd == "!getip"      || cmd=="!gi" ) return CmdGetip(cmd_line,conn);
			if( cmd == "!gethost"    || cmd=="!gh" ) return CmdGethost(cmd_line,conn);
			if( cmd == "!getinfo"    || cmd=="!gn" ) return CmdGetinfo(cmd_line,conn);
			if( cmd == "!help"       || cmd=="!?"  ) return CmdHelp(cmd_line,conn);
			if( cmd == "!ops"        || cmd=="!oc" ) return CmdBroadcast(cmd_line,conn,eUC_OPERATOR,eUC_MASTER);
			if( cmd == "!regs"                     ) return CmdBroadcast(cmd_line,conn,eUC_REGUSER,eUC_MASTER);
			if( cmd == "!regpasswd"  || cmd=="!rp" ) return CmdRegPasswd ( cmd_line, conn);
			if( cmd == "!regdisable" || cmd=="!r0" ) return CmdRegDisable( cmd_line, conn);
			if( cmd == "!regenable"  || cmd=="!r1" ) return CmdRegEnable ( cmd_line, conn);
			if( cmd == "!hideme"     || cmd=="!hm" ) return CmdHideMe    (cmd_line, conn);
			if( cmd == "!hidekick"   || cmd=="!hk" ) return CmdHideKick  (cmd_line, conn);
			if( cmd == "!unhidekick" || cmd=="!uhk") return CmdUnHideKick(cmd_line, conn);
			if( cmd == "!gag"                      ) return CmdGag       (cmd_line, conn);
			if( cmd == "!ungag"                    ) return CmdUnGag     (cmd_line, conn);

			break;
		default: return 0; break;
	}
	return 0;
}

/** act on usr's command */
int cDCConsole::UsrCommand(const string & str, cConnDC * conn)
{
	stringstream cmd_line(str);
	string cmd;
	cmd_line >> cmd;

	switch(conn->mpUser->mClass)
	{
		case eUC_MASTER:
		case eUC_ADMIN:
		case eUC_CHEEF:
		case eUC_OPERATOR:
		case eUC_VIPUSER:
		case eUC_REGUSER:
		case eUC_NORMUSER:
			if( cmd == "+passwd" ) return CmdRegMyPasswd( cmd_line, conn);
			if( cmd == "+help"     ) return CmdHelp(cmd_line , conn);
			if( cmd == "+rules"     ) return CmdRules(cmd_line , conn);
			if( cmd == "+pravidla"     ) return CmdRules(cmd_line , conn);
			if( cmd == "+faq"     ) return CmdFaq(cmd_line , conn);
		break;
		default: break;
	}

	for( int i = 0; i < mTriggers.Size(); ++i )
	{
		if(
			mTriggers[i]->mMinClass <= conn->mpUser->mClass &&
			cmd == mTriggers[i]->mCommand
		)
		{
			if(Log(3)) LogStream() << "trigger found " << cmd << endl;
			return mTriggers[i]->DoIt (cmd_line, conn, mS);
		}
	}
	return 0;
}


/** get user's ip */
int cDCConsole:: CmdGetip(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;

	while(!cmd_line.eof())
	{
		cmd_line >> s;
		user = mS.FindUserNick(s);
		if(user && user-> mxConn )
			os << "User: " << s << " IP: " << user->mxConn->AddrIP() << endl;
		else
			os << "User: " << s << " not found in nicklist." << endl;
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;

}

/** get user's host */
int cDCConsole::CmdGethost(stringstream &cmd_line , cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;
	while(!cmd_line.eof())
	{
		cmd_line >> s;
		user = mS.FindUserNick(s);
		if(user && user->mxConn)
			os << "User: " << s << " Host: " << user->mxConn->AddrHost() << endl;
		else     os << "User: " << s << " not found in nicklist." << endl;
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** get user's host and ip */
int cDCConsole::CmdGetinfo(stringstream &cmd_line , cConnDC *conn )
{
	ostringstream os;
	string s;
	cUser * user;
	while(!cmd_line.eof())
	{
		cmd_line >> s;
		user = mS.FindUserNick(s);
		if(user && user->mxConn)
			os << "User: " << s
				 << " IP: " << user->mxConn->AddrIP()
				 << " Host: " << user->mxConn->AddrHost() << endl;
		else     os << "User: " << s << " not found in nicklist." << endl;
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** unban ip or host */
int cDCConsole::CmdUnban(stringstream & cmd_line, cConnDC * conn,int unbantype )
{
	ostringstream os;
	string s, cmd("!");
	int n;

	if(unbantype == eBT_HOST ) cmd+="host";
	if(unbantype == eBT_NICK ) cmd+="nick";
	if(unbantype == eBT_IP   ) cmd+="ip"  ;
	cmd+="unban";
	cmd_line >> s;
	if(!s.size())
	{
		os << "Use " << cmd << " <item>";
		mS.DCPublicHS(os.str().data(),conn);
		return 1;
	}

	string ip, nick, host;
	if(unbantype == eBT_HOST ) host = s;
	if(unbantype == eBT_NICK ) nick = s;
	if(unbantype &  eBT_IP   ) ip   = s;
	
	n = mS.mBanList.RemoveBan(ip,nick, host, unbantype);

	if(n <= 0)
	{
		os << "Not found " << s << " in banlist.";
		mS.DCPublicHS(os.str().data(),conn);
		return 1;
	}
	os << "Removed " << s << endl;
//		 << "code was: " << br->mCode << endl
//		 << "nick was: " << br->mNick << endl
//		 << "op was: "   << br->mOp << endl;
//	delete br;
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** ban ip or host */
int cDCConsole::CmdBan(stringstream & cmd_line, cConnDC * conn, bool perm, int bantype)
{
	unsigned u=0;
	string s,s2;
	string cmd("!");
	ostringstream os, usage;

	if(!perm) cmd+="temp";
	if(bantype == eBT_HOST) cmd+= "host";
	if(bantype == eBT_NICK) cmd+= "nick";
	cmd+="ban";
	
	// write usage
	usage << "Usage : ";
	if(perm) usage << cmd << " <ip> [<reason>]" << endl;
	else usage << cmd << " <ip> <period> [<reason>]" << endl;
	usage << "\t<ip> is the banned ip address" << endl;
	if(!perm) usage << "\t<period> is for how long the ban is valid, it is a number followed by a unit one of [smhdWMY]" << endl;
	usage << "\t<reason> is the ban reason, this is not obligatory, if you've just kicked a user with this ip address" << endl
				<< "See !help for maybe more detais; see: !ban !tempban" << endl;

	cmd_line >> s;

	if(!perm)
	{
		cmd_line >> s2;
		if(!(u = mS.Str2Period(s2, os)))
		{
			os << usage.str();
			mS.DCPublicHS(os.str(),conn);
			return 1;
		}
		s2="";
	} // if !perm
	
	getline(cmd_line, s2); // get parameters n-nulber, c-unit, s2 code
	
	cKick kick;
	string empty;
	int added;
	if(
		bantype == eBT_NICKIP &&
		mS.mBanList.FindKick(kick, 300, s, conn->mpUser->mNick, true)
		)
	{
		if(s2.size())
		{
			kick.mReason += "\n";
			kick.mReason += s2;
		}
		added = mS.mBanList.AddBan(kick.mIP, kick.mNick, empty, conn, u, kick.mReason, bantype );
		os << "Found your kick with same ip" << endl
			<< "Before: " << time(NULL)-kick.mTime << " seconds" << endl
			<< "Nick: " << kick.mNick << endl
			<< "Reason: " << kick.mReason << endl;
	}
	else
	{
		if(bantype == eBT_NICKIP) bantype = eBT_IP;
		if(!s2.size())
		{
			os << "No corresponding kick found, please give a ban reason!" << endl << usage.str();
			mS.DCPublicHS(os.str(),conn);
			return 1;
		}
		string ip,host,nick;
		if(bantype & eBT_HOST) host=s;
		if(bantype & eBT_NICK) nick=s;
		if(bantype & eBT_IP  ) ip  =s;

		added = mS.mBanList.AddBan(ip , nick ,host, conn, u, s2, bantype );
	}

	if(added)
	{
		os << "Added " << s << " to the banlists:";
		if( added & eBT_IP   ) os << "ip ";
		if( added & eBT_NICK ) os << "nick ";
		if( added & eBT_HOST ) os << "host ";
		os << ".";
		mS.DCPublicHS(os.str(),conn);
		return 1;
	}

	os << "Not added :(";
	mS.DCPublicHS(os.str(),conn);
	return 1;
}

/** quit program */
int cDCConsole::CmdQuit(stringstream &, cConnDC * conn)
{
	ostringstream os;
	os << "Stopping Hub.";
	mS.DCPublicHS(os.str(),conn);
	mS.stop();
	return 1;
}

/** set variable */
int cDCConsole::CmdSet(stringstream & cmd_line, cConnDC * conn)
{
	string var,val;
	ostringstream os;
	cmd_line >> var;
	cConfigItemBase *ci = mS.mC[var];
	if( !ci )
	{
		os << "Undefined variable: " << var;
		mS.DCPublicHS(os.str(),conn);
		return 1;
	}
	cmd_line >> *ci;
	os << "OK, " << var << " is now: " << *ci;
	mS.DCPublicHS(os.str(),conn);
	return 1;
}

/** show alll variables along with their values */
int cDCConsole::CmdGetconfig(stringstream & , cConnDC * conn)
{
	ostringstream os;
	cConfigBase::tIHIt it;
	os << "Config:" << endl;
	for(it = mS.mC.mhItems.begin();it != mS.mC.mhItems.end(); it++)
		os << setw(20) << it->first << " = " << *(it->second) << endl;
	mS.DCPublicHS(os.str(),conn);
	return 1;
}

/** send help message corresponding to connection */
int cDCConsole::CmdHelp(stringstream &, cConnDC * conn)
{
	if(!conn || !conn->mpUser) return 1;
	string file;
	switch(conn->mpUser->mClass)
	{
		case eUC_MASTER : file = ".verlihub/help_master"; mS.SendFileHS(conn, file);
		case eUC_ADMIN :file = ".verlihub/help_admin"; mS.SendFileHS(conn, file);
		case eUC_CHEEF : file = ".verlihub/help_cheef"; mS.SendFileHS(conn, file);
		case eUC_OPERATOR: file = ".verlihub/help_op"; mS.SendFileHS(conn, file);
		case eUC_VIPUSER:
		case eUC_REGUSER: file = ".verlihub/help_reg"; mS.SendFileHS(conn, file);
		case eUC_NORMUSER: file = ".verlihub/help_usr"; mS.SendFileHS(conn, file);
		default: break;
	}
	return 1;
}

/** send rules */
int cDCConsole::CmdRules(stringstream & , cConnDC * conn)
{
	string file(".verlihub/rules");
	mS.SendFileHS(conn, file);
	return 1;
}

/** send faq */
int cDCConsole::CmdFaq(stringstream & , cConnDC * conn)
{
	string file(".verlihub/faq");
	mS.SendFileHS(conn, file);
	return 1;
}

/** broadcast a message to pm of all */
int cDCConsole::CmdBroadcast(stringstream & cmd_line, cConnDC * conn, int cl_min, int cl_max)
{
	string start, end, str;
	ostringstream ostr;
	// save the position
	int pos = 1+cmd_line.tellg();

	// test for existence of parameter
	cmd_line >> str;
	if(! str.size())
	{
		ostr << "See !help; use with parameter" << endl;
                mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	//  $To: <othernick> From: <nick> $<<nick>> <message>
	start = "$To: ";
	end = " From: ";
	//end+= conn->mpUser->mNick;
	end+= mS.mC.hub_security;
	end+= " $<";
	end+= conn->mpUser->mNick;
	end+= "> ";
	end+=cmd_line.str().substr(pos);

	mS.SendToAllWithNick(start,end, cl_min, cl_max);
	mS.LastBCNick = conn->mpUser->mNick;
	return 1;
}

int cDCConsole::CmdRegNewUser (stringstream & cmd_line, cConnDC * conn)
{
	string nick;
	ostringstream ostr;
	int cl=1;
	cmd_line >> nick >> cl;
	if(!nick.size())
	{
		ostr << "Usage !regnewuser <nick> [<class>=1]" << endl << "See !help" << endl;
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}
	if( cl >= conn->mpUser->mClass - 1 )
	{
		ostr << "You can create/modify only lower users than class " << conn->mpUser->mClass << ".";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	int ret=mS.mR.AddRegUser(nick, conn, cl);
	switch(ret)
	{
		case 0:
			ostr << "User added, tell him to change his password.";
			mS.DCPublicHS(ostr.str(), conn);
			break;
		case 1:
			ostr << "User alreday registered.";
			mS.DCPublicHS(ostr.str(), conn);
			break;
		default:
			ostr << "Error while adding user.";
			mS.DCPublicHS(ostr.str(), conn);
		break;
	}

	cUser *user;
	user = mS.FindUserNick(nick);
	if(user && user->mxConn)
	{
		ostr.str(mS.mEmpty);
		ostr << "You have been registered, please set up your password NOW using command +passwd your_new_passwd.";
		mS.DCPublicHS(ostr.str(), user->mxConn);
	}
	return 1;
}

int cDCConsole::CmdRegDelUser (stringstream & cmd_line, cConnDC * conn)
{
	string str;
	ostringstream ostr;
	cmd_line >> str;
	if(!str.size())
	{
		ostr << "Try !help";
		mS.DCPublicHS(ostr.str(), conn);
		return 0;
	}
	return 0;
}

int cDCConsole::CmdRegPurge   (stringstream & cmd_line, cConnDC * conn)
{
	string str;
	ostringstream ostr;
	cmd_line >> str;
	if(!str.size())
	{
		ostr << "Try !help";
		mS.DCPublicHS(ostr.str(), conn);
		return 0;
	}
	return 0;
}

int cDCConsole::CmdRegClass   (stringstream & cmd_line, cConnDC * conn)
{
	string str;
	ostringstream ostr;
	int cl = 1;
	cmd_line >> str >> cl;
	string cls;
	string field ("class");

	if(!str.size())
	{
		ostr << "Try !help";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}
	cls += ('0'+cl);

	cRegUserInfo ui;

	if(
		cl >= conn->mpUser->mClass ||
		!mS.mR.FindRegInfo(ui,str) ||
		ui.mClass >= conn->mpUser->mClass ||
		!mS.mR.SetVar(str,field,cls)
		)
	{
		ostr << "Error";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	ostr << "Ok, user " << str << " has modified class now.";
	mS.DCPrivateHS(ostr.str(), conn);

	return 1;

	return 1;
}

int cDCConsole::CmdRegPasswd  (stringstream & cmd_line, cConnDC * conn)
{
	string str;
	ostringstream ostr;
	cmd_line >> str;
	string field ("pwd_change");
	string value ("1");
	
	if(!str.size())
	{
		ostr << "Try !help";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	cRegUserInfo ui;

	if(
		!mS.mR.FindRegInfo(ui,str) ||
		ui.mClass >= conn->mpUser->mClass ||
		!mS.mR.SetVar(str,field,value)
		)
	{
		ostr << "Error";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	ostr << "Ok, user " << str << " can change his passwd now";
	mS.DCPrivateHS(ostr.str(), conn);
	cUser *other = mS.FindUserNick(str);
	if(other && other->mxConn)
	{
		ostr.str(mS.mEmpty);
		ostr << "Change your password, use command +passwd";
		mS.DCPrivateHS(ostr.str(), other->mxConn, &conn->mpUser->mNick);
	}
	
	return 1;
}

int cDCConsole::CmdRegEnable  (stringstream & cmd_line, cConnDC * conn)
{
	string str;
	ostringstream ostr;
	cmd_line >> str;
	string field("enabled");
	string value("1");

	if(!str.size())
	{
		ostr << "Try !help";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	cRegUserInfo ui;

	if(
		!mS.mR.FindRegInfo(ui,str) ||
		ui.mClass >= conn->mpUser->mClass ||
		!mS.mR.SetVar(str,field, value)
		)
	{
		ostr << "Error";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	ostr << "Ok, user " << str << " can login again";
	mS.DCPublicHS(ostr.str(), conn);

	return 1;
}

int cDCConsole::CmdRegDisable (stringstream & cmd_line, cConnDC * conn)
{
	string str;
	ostringstream ostr;
	cmd_line >> str;
	string field("enabled");
	string value("0");
	
	if(!str.size())
	{
		ostr << "Try !help";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	cRegUserInfo ui;

	if(
		!mS.mR.FindRegInfo(ui,str) ||
		ui.mClass >= conn->mpUser->mClass ||
		!mS.mR.SetVar(str,field,value)
		)
	{
		ostr << "Error";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	ostr << "Ok, user " << str << " cannot login now.";
	mS.DCPublicHS(ostr.str(), conn);

	return 1;
	return 1;
}

int cDCConsole::CmdRegMyPasswd(stringstream & cmd_line, cConnDC * conn)
{
	string str;
	int crypt = 0;
	ostringstream ostr;
	cRegUserInfo ui;

	if(!mS.mR.FindRegInfo(ui,conn->mpUser->mNick))
		return 0;

	if(!ui.mPwdChange)
	{
		ostr << "You are not allowed to change your password now. Ask any OP.";
		mS.DCPrivateHS(ostr.str(),conn);
		return 1;
	}

	cmd_line >> str >> crypt;
	if(str.size() < 6)
	{
		ostr << "Minimum password length is 6 characters, retry..";
		mS.DCPrivateHS(ostr.str(),conn);
		return 1;
	}
	if(mS.mR.ChangePwd(conn->mpUser->mNick, str,crypt))
	{
		ostr << "Password not changed :(.";
		mS.DCPrivateHS(ostr.str(),conn);
		return 1;
	}

	ostr << "Password updated successfully.";
	mS.DCPrivateHS(ostr.str(),conn);
	return 1;
}

/** send kick/drop user */
int cDCConsole::CmdKick(stringstream & cmd_line, cConnDC * conn, bool drop)
{
	string nick, reason;
	ostringstream ostr;
	
	cmd_line >> nick;
	getline(cmd_line,reason);

	if(!drop)
	{
		if(!reason.size())
		{
			ostr << "Usage !kick <nick> <reason>" << endl;
			mS.DCPublicHS(ostr.str(), conn);
			return 1;
		}
	}

	cUser *other= mS.FindUserNick(nick);
	if(!other || !other->mxConn)
	{
		ostr << "User " << nick << " is not in userlist" << endl;
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	if(other->mClass >= conn->mpUser->mClass || other->mProtectFrom >= conn->mpUser->mClass)
	{
		ostr << "User " << nick << " can't be disconnected by you (privileges)" << endl;
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	mS.mBanList.AddKick(other->mxConn, conn, NULL); // drop
	mS.mBanList.AddKick(other->mxConn, conn, &reason); // message

	if(!drop)
	{
		ostr << "<" << conn->mpUser->mNick << "> " << conn->mpUser->mNick << " is kicking " << nick << " because: " << reason;
		string msg = ostr.str();
		mS.SendToAll(msg, eUC_NORMUSER, eUC_MASTER);
		ostr.str(mS.mEmpty);
		ostr << "You are being kicked because: " << reason;
		mS.DCPrivateHS(ostr.str(), other->mxConn, &conn->mpUser->mNick);
	}
	other->mxConn->CloseNice(1000);


	return 1;
}

/** banlist */
int cDCConsole::CmdBanList(stringstream & cmd_line, cConnDC * conn, int bantype, bool filter)
{
	string ipnick;
	ostringstream omsg;
	cmd_line >> ipnick;
	if(filter)
	{
		if(!ipnick.size())
		{
			omsg << "use with a paramater";
			mS.DCPublicHS(omsg.str(), conn);
			return 1;
		}
	}
	mS.mBanList.GetBanList(conn, bantype, filter? &ipnick :NULL);
	return 1;
}

/** get list of users with given ip */
int cDCConsole::CmdWhoIP(stringstream & cmd_line, cConnDC * conn)
{
	string ip;
	cmd_line >> ip;
	ostringstream omsg;
	
	if(!ip.size())
	{
		omsg << "use with a paramater";
		mS.DCPublicHS(omsg.str(), conn);
		return 1;
	}

	string separator("\r\n\t");
	string userlist;

	int cnt = mS.WhoIP(ip, userlist, separator, true);

	if(!cnt) omsg << "No users has " << ip;
	else omsg << "Users with ip " << ip << ":\n\r\t" << userlist << "Total: " << cnt;
	mS.DCPrivateHS(omsg.str(), conn);
	return 1;
}

/** make a secret user/op */
int cDCConsole::CmdHideMe(stringstream & cmd_line, cConnDC * conn)
{
	int cls = -1;
	cmd_line >> cls;
	ostringstream omsg;
	if(cls < 0)
	{
		omsg << "Use !hidmeme <class>\n\r <class> the maximum class uf users, that is not allowed to see you." << endl;
		mS.DCPublicHS(omsg.str(),conn);
		return 1;
	}
	if(cls > conn->mpUser->mClass) cls = conn->mpUser->mClass;
	conn->mpUser->mHideKicksForClass = cls;
	omsg << "OK, your kicks are hidden for all lesser than " << cls << " users.";
	mS.DCPublicHS(omsg.str(),conn);
	return 1;
}


/*!
    \fn cDCConsole::CmdUserLimit(stringstream & cmd_line, cConnDC * conn);
    usage: !userlimit <max_users> [<minutes>=60]
 */
int cDCConsole::CmdUserLimit(stringstream & cmd_line, cConnDC * conn)
{
	string str;
	ostringstream ostr;
	int minutes = 60, maximum = -1;
	cmd_line >> maximum >> minutes;

	if( maximum < 0 )
	{
		ostr << "Try !help (usage !userlimit <max_users> [<minutes>=60])";
		mS.DCPublicHS(ostr.str(), conn);
		return 1;
	}

	// 60 steps at most
	cInterpolExp *fn = new
		cInterpolExp(mS.mC.max_users, maximum, (60*minutes) / mS.timer_serv_period ,(6*minutes) / mS.timer_serv_period);
	mS.mTmpFunc.push_back((cTempFunctionBase *)fn);

	ostr << "Starting the max_users change start: "
		<< mS.mC.max_users << " end: " << maximum
		<< " duration: " << minutes << " minutes";
	mS.DCPrivateHS(ostr.str(), conn);

	return 1;
}

/** gag users until reconnect
	usage !gag <nick> ...
*/
int cDCConsole::CmdGag(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;

	while(!cmd_line.eof())
	{
		cmd_line >> s;
		user = mS.FindUserNick(s);
		if(user && user-> mxConn && user->mClass < conn->mpUser->mClass)
		{
			os << "User: " << s << " is being gaged from chat." << endl;
			user->mGag = true;
		}
		else
		{
			os << "User: " << s << " not found or not being gaged." << endl;
		}
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** ungag user
	usage: !ungag <nick> ...
*/
int cDCConsole::CmdUnGag(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;

	while(!cmd_line.eof())
	{
		cmd_line >> s;
		user = mS.FindUserNick(s);
		if(user && user-> mxConn && user->mClass < conn->mpUser->mClass)
		{
			os << "User: " << s << " being ungaged.:)" << endl;
			user->mGag = false;
		}
		else
			os << "User: " << s << " not found in nicklist or you can't ungag him." << endl;
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** class user (change temporarily his class)
	!class <nick> <new_class_0_to_5>
*/
int cDCConsole::CmdClass(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;
	int nclass = 3, oclass, mclass = conn->mpUser->mClass;

	cmd_line >> s >> nclass;

	if(!s.size() || nclass < 0 || nclass > 5 || nclass >= mclass)
	{
		os << "Try !help, usage !class <nick> [<class>=3]" << endl
			<< "class max is " << mclass << endl;
		mS.DCPublicHS(os.str().data(),conn);
		return 1;
	}


	user = mS.FindUserNick(s);

	if( user && user->mxConn )
	{
		oclass = user->mClass;
		if( oclass < mclass )
		{
			os << "User: " << s << " temp changing class to " << nclass << endl;
			user->mClass = (tUserCl) nclass;
			#warning todo if some changes in operators etc, send corresponding infos
		}
		else
		{
			os << "You don't have privileges to change class of " << s << "." << endl;
		}
	}
	else
	{
		os << "User: " << s << " not found in nicklist." << endl;
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** hidekick users until reconnect
	usage !hidekick <nick> ...
*/
int cDCConsole::CmdHideKick(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;

	while(!cmd_line.eof())
	{
		cmd_line >> s;
		user = mS.FindUserNick(s);
		if(user && user-> mxConn && user->mClass < conn->mpUser->mClass)
		{
			os << "User: " << s << " kicks are now hidden." << endl;
			user->mHideKick = true;
		}
		else
		{
			os << "User: " << s << " not found." << endl;
		}
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** unhidekick user
	usage: !unhidekick <nick> ...
*/
int cDCConsole::CmdUnHideKick(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;

	while(!cmd_line.eof())
	{
		cmd_line >> s;
		user = mS.FindUserNick(s);
		if(user && user-> mxConn && user->mClass < conn->mpUser->mClass)
		{
			os << "User: " << s << " will show kick messages to chat" << endl;
			user->mHideKick = false;
		}
		else
			os << "User: " << s << " not found in nicklist (or no rights)." << endl;
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

/** protect user against kicks
	!protect <nick> [<against_class>=your_class-1]
*/
int cDCConsole::CmdProtect(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;
	string s;
	cUser * user;
	int oclass, mclass = conn->mpUser->mClass, nclass = mclass -1;
	if(nclass > 5) nclass = 5;

	cmd_line >> s >> nclass;

	if(!s.size() || nclass < 0 || nclass > 5 || nclass >= mclass)
	{
		os << "Try !help, usage !protect <nick> [<againstclass>=your_class-1]" << endl
			<< "class max is " << mclass-1 << endl;
		mS.DCPublicHS(os.str().data(),conn);
		return 1;
	}


	user = mS.FindUserNick(s);

	if( user && user->mxConn )
	{
		oclass = user->mClass;
		if( oclass < mclass )
		{
			os << "User: " << s << " temp changing protection to " << nclass << endl;
			user->mProtectFrom = nclass;
		}
		else
			os << "You don't have privileges to protect of " << s << "." << endl;
	}
	else
	{
		os << "User: " << s << " not found in nicklist." << endl;
	}
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

int cDCConsole::CmdReload(stringstream &cmd_line, cConnDC *conn)
{
	ostringstream os;

	os << "Reloading triggers " << endl;
	#warning todo create reload function
	mTriggers.LoadAll();
	mS.DCPublicHS(os.str().data(),conn);
	return 1;
}

};

