/***************************************************************************
                          cdcproto.cpp  -  description
                             -------------------
    begin                : Wed Jul 2 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cdcproto.h"
#include "cserverdc.h"
#include "cconndc.h"
#include "cmessagedc.h"

#define CHECK_IP_CTM 1
#define CHECK_NICK_RCTM 1
#define CHECK_IP_ASRCH 0
#define CHECK_NICK_PSRCH 1
#define CHECK_NICK_SR 1

namespace nDirectConnect
{

typedef enum
{
	eSF_OPPLUS=1 // OpPlus commands
}tSupportFeature;

cDCProto::cDCProto(cServerDC &serv):mS(serv)
{
	char *errptr;
	int erroffset;
	mKickChatPattern  = NULL;
	mKickChatPatternE = NULL;
	mKickBanPattern   = NULL;
	mKickBanPatternE  = NULL;

	mKickChatPattern = pcre_compile("^(\\S+) is kicking (\\S+) [bB]ecause: (.*)$",0,(const char **)&errptr,&erroffset,NULL);
	if(!mKickChatPattern) throw "error in kickchatpattern";
//	mKickChatPatternE = pcre_study(mKickChatPattern, 0, (const char **)&errptr);
//	if(!mKickChatPatternE) throw "error in kickcahtpattern in tusy";

	mKickBanPattern = pcre_compile("_[bB][aA][nN]_(\\d+[smhdwySHMDWY]?)?",0,(const char **)&errptr,&erroffset,NULL);
	if(!mKickChatPattern) throw "error in kickbanpattern";
//	mKickBanPatternE = pcre_study(mKickBanPattern, 0, (const char **)&errptr);
//	if(!mKickBanPatternE) throw "error in kickbanpattern in tusy";

}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_ValidateNick(cMessageDC *msg, cConnDC *conn)
{
	if(msg->SplitChunks()) return -1;
	string &nick = msg->ChunkString(eCH_1_PARAM);
	static string omsg;
	ostringstream os;

	// log this event
	if(conn->Log(3)) conn->LogStream() << "User " << nick << " tries to login" << endl;
	
	// test valid ip and nick, close conn eventually
	if(!mS.ValidateUser(conn, nick))
	{
//		omsg="$ValidateDenide";
//		conn->Send(omsg);
		conn->CloseNice(1000);
		return -1;
	}
	int limit=mS.mC.max_users; // user limit
//	cRegUserInfo ui;//mS.CfgGetRegInfo(nick);
//	int reg_user;

	// calculate user's limit
	if(conn->mRegInfo.mClass >= eUC_REGUSER ) limit+=mS.mC.max_extra_regs;
	if(conn->mRegInfo.mClass >= eUC_OPERATOR) limit+=mS.mC.max_extra_ops;
	if(conn->mRegInfo.mClass >= eUC_ADMIN   ) limit+=mS.mC.max_extra_admins;

	// Check the max_users limit
	if(mS.mUserCount >= limit)
	{
		os << "Sorry, the hub is full(" << mS.mUserCount << "), try again later...";
		if(conn->Log(2))
			conn->LogStream() << "Hub is full (" << mS.mUserCount << "/"<< limit << "), closing." << endl;
		mS.ConnCloseMsg(conn,os.str(),1000);
		return -1;
	}else
	{
		conn->SetLSFlag(eLS_ALOWED);
		mS.mUserCount ++;
	}

	// send hubname
	omsg="$HubName ";
	omsg+= mS.mC.hub_name;
	conn->Send(omsg);

	if(conn->mRegInfo.mClass && !conn->mRegInfo.mPwdChange)
	{
		omsg="$GetPass";
		conn->Send(omsg);
	}
	else
	{ // the user doesn't need password, so he's already in
		mS.DCHello(nick, conn);
		conn->SetLSFlag(eLS_PASSWD);
	}
	try
	{
		conn->SetUser(new cUser(nick));
	}catch(...)
	{
		if(mS.ErrLog(2)) mS.LogStream() << "Unhandled exception in cServerDC::DC_ValidateNick" << endl;
		omsg="Sorry :(";
		if(conn->Log(2)) conn->LogStream() << "Fatal error, in SetUser closing.." << endl;
		mS.ConnCloseMsg(conn,omsg,1000);
		return -1;
	}

	conn->SetLSFlag(eLS_VALNICK);
	//conn->SetTimeOut(eTO_MYINFO,20);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_Key(cMessageDC * , cConnDC * conn)
{
	//TODO 2DO verify the key, close connection on bad key
	conn->SetLSFlag(eLS_KEYOK);
	conn->ClearTimeOut(eTO_KEY);
	conn->mT.key = ::time(NULL);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_MyPass(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	string &pwd=msg->ChunkString(eCH_1_PARAM);

	string omsg;
	
	if(!conn->mpUser)
	{
		omsg = "Bas Login sequence, you must provide valid nick first.";
		if(conn->Log(1))conn->LogStream() << "Mypass before validatenick" << endl;
		return -1;
		mS.ConnCloseMsg(conn,omsg,1000);
	}
	if(conn->mpUser->CheckPwd(pwd)) // check the password
	{
		conn->SetLSFlag( eLS_PASSWD );
		// setup the user class..
		conn->mpUser->Register();
		mS.mR.Login(conn, conn->mpUser->mNick);
		mS.DCHello(conn->mpUser->mNick,conn);
		// if op .. send LoggedIn and Oplist
		if(conn->mpUser->mClass >= eUC_OPERATOR)
		{
			omsg = "$LogedIn ";
			omsg+= conn->mpUser->mNick;
			conn->Send(omsg);
		}
	}
	else // wrong password
	{
		omsg="$BadPass";
		conn->Send(omsg);
		omsg="You provided wrong password, 30s ban.";
		mS.mBanList.AddBan(conn->mAddrIP, conn->mpUser->mNick, mS.mEmpty ,NULL,30, string("pass"), eBT_IP | eBT_NICK);
		mS.mR.LoginError(conn, conn->mpUser->mNick);
		if(conn->Log(2)) conn->LogStream() << "Wrong password, ban " << 30 <<"s, closing" << endl;
		mS.ConnCloseMsg(conn,omsg,2000);
		return -1;
	}
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_Version(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	conn->SetLSFlag( eLS_VERSION );
	string &version=msg->ChunkString(eCH_1_PARAM);
	if(conn->Log(3)) conn->LogStream() << "Version:" << version << endl;
	if(0)           // 2DO TODO check the version, store it
	{
		// wrong version message
		// disconnect
	}
	conn->mVersion=version;
	return 1;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_GetNickList(cMessageDC * , cConnDC * conn)
{
	if(!conn) return -1;
	if(!conn->GetLSFlag(eLS_MYINFO) && mS.mC.nicklist_on_login)
	{
		conn->mSendNickList = true;
		return 0;
	}
	return NickList(conn);
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_MyINFO(cMessageDC * msg, cConnDC * conn)
{
	string cmsg;
	// server gets this once on login, and then yet many times
	if(msg->SplitChunks())
	{
		if(conn->Log(2)) conn->LogStream() << "MyINFO syntax error, closing" << endl;
		mS.ConnCloseMsg(conn, cmsg, 4000);
		return -1;
	}
	string &nick=msg->ChunkString(eCH_MI_NICK);

	// this can't happen without having created user object
	if(!conn->mpUser)
	{
		cmsg = "bad login sequence";
		if(conn->Log(2)) conn->LogStream() << "Myinfo without nick " << nick << endl;
		mS.ConnCloseMsg(conn, cmsg,1000);
		if(mS.ErrLog(0)) mS.LogStream() << "Myinfo without nick " << nick << endl;
		return -1;
	}

	// check syntax a bit

	// check nick
	if(nick != conn->mpUser->mNick)
	{
		cmsg = "Wrong MyINFO";
		if(conn->Log(1)) conn->LogStream() << "Claims to be someone else in MyINFO" << endl;
		mS.ConnCloseMsg(conn, cmsg, 1500);
		return -1;
	}

	// verify conditions
	//    minimal share and maximal
	string &str_share=msg->ChunkString(eCH_MI_SIZE);
	if(str_share.size() > 18) // that is too much
	{
		conn->CloseNow();
		return -1;
	}
	long long share,shareB;
	shareB = strtoll(str_share.data(),NULL,10);
	share  = shareB/(1024*1024);
  // calculate minimax
	long long min_share=mS.mC.min_share;
	long long max_share=mS.mC.max_share;
	if(conn->mpUser->mClass >= eUC_VIPUSER )
	{
		min_share=mS.mC.min_share_reg;
		max_share=mS.mC.max_share_reg;
	}
	if(conn->mpUser->mClass >= eUC_OPERATOR) min_share=mS.mC.min_share_ops;

	if(share < min_share || share > max_share)
	{
		char str[256];
		memset(str,0,sizeof(str));
		sprintf(str,"Minimum share size is: %lld MB and Maximum is: %lld MB..",min_share,max_share);
		cmsg = str;
		if(conn->Log(2))
			conn->LogStream() << "Share limit."<< endl;//min_share << "<=" << str_share << "<=" << max_share << "." << endl;
		mS.ConnCloseMsg(conn, cmsg, 4000);
		return -1;
	}

	// update totalshare
	mS.mTotalShare -= conn->mpUser->mShare;
	conn->mpUser->mShare = shareB;
	mS.mTotalShare += conn->mpUser->mShare;
	
	if(0) // not verified
	{
		cmsg = "not accepted";
		mS.ConnCloseMsg(conn, cmsg, 1000);
		return -1;
	}

  // keep it
	conn->mpUser->mMyINFO=msg->ChunkString(eCH_MI_INFO);
	
	// login or send to all
	if(conn->mpUser->mInList)
	{
		if(mS.MinDelay(conn->mpUser->mT.info,30))
		{
			if(!mS.mC.delayed_myinfo)
				mS.SendToAll(msg->mStr, eUC_NORMUSER, eUC_MASTER);
			else
				mS.mSendAllBuf+=msg->mStr+"|";
		}
	}
	else
	{
		// note, we got it
		conn->SetLSFlag(eLS_MYINFO);
		// if all right, add user to userlist, if not yet there
		mS.DoUserLogin(conn);
		if(!conn->mpUser->mInList) return -1;
		conn->mpUser->mT.login=time(NULL);
	}
	
	conn->ClearTimeOut(eTO_MYINFO);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_GetINFO(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;

	// check if he's in -> this is done by filter
	if(!conn->mpUser || !conn->mpUser->mInList)
		return -1; // for sure
	
	string buf;
	string &str=msg->ChunkString(eCH_GI_OTHER);
	
	cUser *other = mS.FindUserNick ( str );

	// check if user found
	if(!other )
	{
		if(str != mS.mC.hub_security)
		{
			buf = "$Quit ";
			buf += str ;
			conn->Send(buf);
		}
		return -2;
	}

	// if user just logged in ignore it, conn is dcgui, and already one myinfo sent
	if(
		conn->mpUser->mT.login < other->mT.login &&
		time(NULL) < (other->mT.login + 60)
		)
		return 0;

	if(mS.mC.optimize_userlist == eULO_GETINFO)
	{
		conn->mpUser->mQueueUL.append(str);
		conn->mpUser->mQueueUL.append("|");
	}
	else
	{

		// create it
		buf = "$MyINFO $ALL ";
		buf.append(other->mNick);
		buf.append(" ");
		buf.append(other->mMyINFO);

		// send it
		conn->Send(buf);
	}
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_To(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	string &str=msg->ChunkString(eCH_PM_TO);
	ostringstream os;

	if(!conn->mpUser) return -1;
	// verify sender's nick
	if(
			msg->ChunkString(eCH_PM_FROM) != conn->mpUser->mNick||
			msg->ChunkString(eCH_PM_NICK) != conn->mpUser->mNick
		)
	{
		if(conn->Log(1)) conn->LogStream() << "Claims to be someone else i PM (" << msg->ChunkString(eCH_PM_FROM) << ")." <<endl;
		conn->CloseNow();
		return -1;
	}
	cTime now;
	now.Get();
	if(0 && conn->mpUser->mFloodPM.Check(now))
	{
		if(conn->Log(1)) conn->LogStream() << "Floods PM (" << msg->ChunkString(eCH_PM_FROM) << ")." <<endl;
		mS.DCPrivateHS("Flooding PM", conn);
		conn->CloseNow();
		return -1;
	}

	// find other user
	cUser *other = mS.FindUserNick ( str );
	if(!other)
	{
//		os << "Sorry, user " << str << "isn't anymore in my userlist.";
//		mS.DCPublicHS(os.str(), conn);
		return -2;
	}

	// log it
	if(mS.Log(5)) mS.LogStream()
		<< "PM from:" << conn->mpUser->mNick
		<< " To: " << msg->ChunkString(eCH_PM_TO) << endl;

	// send it
	if(other->mxConn)
		other->mxConn->Send(msg->mStr);
	else if(str == mS.mC.hub_security)
	{
		other = mS.FindUserNick ( mS.LastBCNick );
		if(other && other->mxConn)
		{
			if(!ParseForCommands(msg->ChunkString(eCH_PM_MSG),conn))
				mS.DCPrivateHS(msg->ChunkString(eCH_PM_MSG),other->mxConn,&conn->mpUser->mNick);
		}
		else
			mS.DCPrivateHS("Hub-security doesn't accept pm's", conn);
	}
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_Chat(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	if(!conn->mpUser) return -2;
	if(!conn->mpUser->mInList) return -3;
	if(conn->mpUser->mGag) return -4;
	stringstream omsg;
	bool send=false;
	// set minimum chat delay
	int delay=1; if(conn->mpUser->mClass >=  eUC_VIPUSER) delay=0;

	// check if nick is ok
	if( (msg->ChunkString(eCH_CH_NICK) != conn->mpUser->mNick) )
	{
		omsg << "You are not " << msg->ChunkString(eCH_CH_NICK) << ".";
		mS.DCPublicHS(omsg.str(),conn);
		if(conn->Log(2))
			conn->LogStream() << "Claims to be " << msg->ChunkString(eCH_CH_NICK) << " in chat." << endl;
		conn->CloseNice(1000);
		return -2;
	}

	// check if delay is ok, yet here
	if(!mS.MinDelay(conn->mpUser->mT.chat,delay))
	{
		omsg << "Minimum delay for chat is: " << delay << "s.";
		mS.DCPublicHS(omsg.str(),conn);
		return 0;
	}

	send = true;

	string &text= msg->ChunkString(eCH_CH_MSG);

	if(ParseForCommands(text, conn)) return 0;

	////////// here is the part that finally distributes messages
	// check message length only for less than vip regs
	if((text.size() > mS.mC.max_chat_msg ) && conn->mpUser->mClass < eUC_VIPUSER)
	{
		omsg << "Too long Chat message, not sending it. Max alowed length is:"
			   << mS.mC.max_chat_msg << " and you are trying: " << text.size() << " characters.";
		mS.DCPublicHS(omsg.str(),conn);
		return 0;
	}

	// count message lines for lower users
	if( conn->mpUser->mClass < eUC_VIPUSER )
	{
		int lines=1;
		unsigned pos=0;
		// the ?:  ... because in the beginning start search from the first char, but then from next after last found
		while(msg->mStr.npos != (pos=msg->mStr.find_first_of("\n",pos?pos+1:0)))
		{
			lines++;
			if(lines > mS.mC.max_chat_lines)
			{
				omsg << "Chat not sent. Maximum lines number in chat is set to: " << mS.mC.max_chat_lines;
				mS.DCPublicHS(omsg.str(),conn);
				return 0;
			}
		}
	}

	// if this is a kick message, process it separately
	int rc=0, ov[30];
	memset(ov,0, sizeof(ov));
	if(
		conn->mpUser->mClass >= eUC_OPERATOR &&
		(rc = pcre_exec(mKickChatPattern, mKickChatPatternE, text.data(), text.size(), 0, 0, ov, 30)) >= 3 &&
		text.compare(0,ov[3], conn->mpUser->mNick) == 0
	)
	{
		// now it is for sure a kick-like message
		string kick_reason(text, ov[6]);
		string nick(text, ov[4],ov[5]-ov[4]);

		cUser *user = mS.FindUserNick(nick);
		// is it a kickable user?
		if(user && user->mxConn && user->mClass <= conn->mpUser->mClass )
		{
			user->mToBan = false;
			// auto kickban
			if((rc=pcre_exec(mKickBanPattern, mKickBanPatternE, kick_reason.data(), kick_reason.size(), 0, 0, ov, 30)) >= 0)
			{
				user->mToBan = true;
				user->mBanReason = kick_reason;
				unsigned u=0;
				if(ov[2] >= 0 )
				{
					string bantime(kick_reason,ov[2],ov[3]-ov[2]);
					ostringstream os;
					if(bantime.size())
						if(!(u=mS.Str2Period(bantime,os)))
							mS.DCPublicHS(os.str(),conn);
				}
				user->mBanTime = u;
			}
			mS.mBanList.AddKick(user->mxConn, conn, &kick_reason);
		}

		if(mS.mC.hide_all_kicks || conn->mpUser->mHideKick ) send = false;
		if(send) mS.SendToAll(msg->mStr, conn->mpUser->mHideKicksForClass ,eUC_MASTER);
		return 0;
	}

	// finally send the message
	if(send) mS.SendToAll(msg->mStr, eUC_NORMUSER, eUC_MASTER);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_Kick(cMessageDC * msg, cConnDC * conn)
{
	stringstream ostr;
	if(msg->SplitChunks()) return -1;
	string &str = msg->ChunkString(eCH_1_PARAM);

	// check rights
	if(!conn->mpUser || conn->mpUser->mClass < eUC_OPERATOR)
	{
		if(conn->Log(1)) conn->LogStream() << "Tries to Kick " << str  << endl;
		ostr << "You are not an operator, thow you can't kick.";
		mS.ConnCloseMsg(conn,ostr.str(),2000);
		return -1;
	}

	// get other user
	cUser *other = mS.FindUserNick ( str );
	if(
		!other ||
		other->mClass > conn->mpUser->mClass ||
		other->mProtectFrom >= conn->mpUser->mClass
	)
	{
		ostr << "Can't kick user " << str << " :(. (maybe he's higher then you, or protected)";
		mS.DCPublicHS(ostr.str(),conn);
		return -2;
	}

	// verify if there si connection otherwise a big bug
	if(!other->mxConn)
	{
		if(mS.ErrLog(1)) mS.LogStream() << "Kicking user, that has no connection and is in userlist: " << str;
		mS.RemoveNick(str);
		return -3;
	}

	// reply to kicker
	ostr << "Kicked user " << str
			 << " IP: " << other->mxConn->AddrIP();
	if(other->mxConn->AddrHost().length())
		ostr << " Host: " << other->mxConn->AddrHost();
	ostr << " .. ;)";

	// temp ban kicked user
	mS.mBanList.AddKick(other->mxConn, conn, NULL);
	if(other->mToBan)
	{
		if(mS.mBanList.AddBan(other->mxConn->AddrIP(),str,mS.mEmpty,  conn, other->mBanTime, other->mBanReason, eBT_IP|eBT_NICK))
			ostr << endl << "and he is being Banned.";
		else
			ostr << endl << "Not Banned.";
	}
	mS.DCPublicHS(ostr.str(),conn);

	// log it
	if(other->mxConn->Log(2))
		other->mxConn->LogStream() << "Kicked by " << conn->mpUser->mNick << ", ban " << mS.mC.tban_kick << "s"<< endl;
	if(conn->Log(3)) conn->LogStream() << "Kicking " << str << endl;

	// close connection - kick
	other->mxConn->CloseNice(4000);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_ConnectToMe(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	if(!conn->mpUser || !conn->mpUser->mInList) return -1;
	string &nick = msg->ChunkString(eCH_CM_NICK);
	ostringstream os;

	// todo kontrola ip
// 	if(!conn->mpUser->mIP4DL.size())
// 		conn->mpUser->mIP4DL = msg->ChunkString(eCH_CM_IP);
// 	else if(conn->mpUser->mIP4DL != msg->ChunkString(eCH_CM_IP))
// 	{
// 		os << "(automatic) CTM ip faking, flooding users; your ip: " << conn->mAddrIP
// 			<< " pretending to be: " << conn->mpUser->mIP4DL
// 			<< " and:" << msg->ChunkString(eCH_CM_IP) << endl;
// 		mS.mBanList.AddBan( conn->mAddrIP, conn->mpUser->mNick, mS.mEmpty, NULL, 0, os.str(), eBT_NICKIP);
// 		conn->CloseNow();
// 	}

	if(mS.mC.check_ctm && conn->mAddrIP != msg->ChunkString(eCH_CM_IP))
	{
		if( conn->Log(1) ) conn->LogStream() << "Claims to be someone else in ConnecToMe" << endl;
		os << "This: " << msg->ChunkString(eCH_CM_IP) << " ins't your ip. Your's is " << conn->mAddrIP;
		mS.ConnCloseMsg(conn, os.str(), 2000);
		return -1;
	}
	cUser *other = mS.FindUserNick ( nick );
	// check nick
	if(!other) return -1;
	if(other->mxConn)
		other->mxConn->Send( msg->mStr );
	else if(nick == mS.mC.hub_security)
	{
		mS.DCPrivateHS("Hub-security doesn't share.",conn);
	}
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_MultiConnectToMe(cMessageDC * , cConnDC * )
{
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_RevConnectToMe(cMessageDC * msg, cConnDC *conn )
{
	if(msg->SplitChunks()) return -1;
	if(!conn->mpUser) return -1;
	ostringstream ostr;
	if(!conn->mpUser) return -1;
	
	// check nick
	if(mS.mC.check_rctm &&msg->ChunkString(eCH_RC_NICK) != conn->mpUser->mNick)
	{
		ostr << "Your nick isn't: " << msg->ChunkString(eCH_RC_NICK) << " but " << conn->mpUser->mNick;
		if(conn->Log(1)) conn->LogStream() << "Claims to be someone else in RevConnectToMe" << endl;
		mS.ConnCloseMsg(conn, ostr.str(), 1500);
		return -1;
	}

	// find and check the other
	string &str = msg->ChunkString(eCH_RC_OTHER);
	cUser *other = mS.FindUserNick ( str );
	if(!other)
	{
		ostr << "User " << str << " not found.";
		//mS.DCPublicHS(ostr.str(),conn);
		return -2;
	}

	// do it
	if(other->mxConn) other->mxConn->Send( msg->mStr );
	else mS.DCPrivateHS("Hub-security doesn't share.",conn);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_Search(cMessageDC * msg, cConnDC * conn)
{
	ostringstream os;
	if(msg->SplitChunks()) return -1;

	// check if user is logged in at least
	if(!conn->mpUser)
	{
		if(conn->Log(1)) conn->LogStream() << "Can't search without user" << endl;
		conn->CloseNow();
		return -1;
	}
	
	// todo kontorly
	// calculate delay & do some verifications
	int delay=10;
	switch(msg->mType)
	{
		case eDC_MSEARCH:
		case eDC_SEARCH:
			delay=mS.mC.int_search;
			if(CHECK_IP_ASRCH && conn->mAddrIP != msg->ChunkString(eCH_AS_IP))
			{
				if(conn->Log(1)) conn->LogStream() << "Claims to be someone else in Active search." << endl;
				os << "Your ip isn't " << msg->ChunkString(eCH_AS_IP) << " but " << conn->mAddrIP << " bye bye.";
				mS.ConnCloseMsg(conn, os.str(),4000);
				return -1;
			}
			break;
		case eDC_MSEARCH_PAS:
		case eDC_SEARCH_PAS:
			delay=int(1.5 * mS.mC.int_search);
			if(CHECK_NICK_PSRCH &&conn->mpUser->mNick != msg->ChunkString(eCH_PS_NICK))
			{
				if(conn->Log(1)) conn->LogStream() << "Claims to be someone else in Passive search." << endl;
				os << "Your nick isn't " << msg->ChunkString(eCH_PS_NICK) << " but " << conn->mpUser->mNick << " bye bye.";
				mS.ConnCloseMsg(conn, os.str(),4000);
				return -1;
			}
			break;
		default: return -5; break;
	}
	
	if(conn->mpUser->mClass >=  eUC_VIPUSER) delay=1;

	// verify the delay
	if(!mS.MinDelay(conn->mpUser->mT.search,delay))
	{
		os << "Minimum search interval is:" << delay << "s";
		mS.DCPublicHS(os.str(),conn);
		return -1;
	}

	// translate MultiSearch to Search
	string omsg;
	if(msg->mType == eDC_MSEARCH)
	{
		omsg="$Search ";
		omsg+=msg->ChunkString(eCH_AS_ADDR);
		omsg+=' ';
		omsg+=msg->ChunkString(eCH_AS_QUERY);
	}
	else omsg=msg->mStr;
	
	// send message finally
	if(mS.mC.delayed_search)
		mS.mSendAllBuf+=omsg+"|";
	else
		mS.SendToAll(omsg, eUC_NORMUSER, eUC_MASTER);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_SR(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks())
		return -1;

	ostringstream os;

	// check the nick
	if(CHECK_NICK_SR && !conn->mpUser || conn->mpUser->mNick != msg->ChunkString(eCH_SR_FROM))
	{
		if(conn->Log(1)) conn->LogStream() << "Claims to be someone else in SR." << endl;
		if(conn->mpUser)
			os << "Your nick isn't " << msg->ChunkString(eCH_SR_FROM)
				 << " but " << conn->mpUser->mNick << " bye bye.";
		mS.ConnCloseMsg(conn, os.str(),4000);
		return -1;
	}
	
	string &str = msg->ChunkString(eCH_SR_TO);
	cUser *other = mS.FindUserNick ( str );
	// check other nick
	if(!other) return -1;
	if(!conn->mpUser || !conn->mpUser->mInList) return -2;

	// cut off the end
	string ostr(msg->mStr,0 ,msg->mChunks[eCH_SR_TO].first - 1);

	// send it
	if(other->mxConn) other->mxConn->Send( ostr );
	//else mS.DCPrivateHS("Hub-security doesn't search.",conn);
	return 0;
}

/** Treat the DC message in a appropriate way */
int cDCProto::DC_OpForceMove(cMessageDC * msg, cConnDC * conn)
//$ForceMove <newIp>
//$To: <victimNick> From: <senderNick> $<<senderNick>> You are being re-directed to <newHub> because: <reasonMsg>
{
	if(msg->SplitChunks()) return -1;
	if(!conn->mpUser->mInList) return -2;
	ostringstream ostr;
	
	string &str = msg->ChunkString(eCH_FM_NICK);
	
	// check rights
	if(!conn->mpUser || conn->mpUser->mClass < eUC_OPERATOR)
	{
		if(conn->Log(1)) conn->LogStream() << "Tries to force move " << str  << endl;
		ostr << "You are not an operator, though you can't force move.";
		mS.ConnCloseMsg(conn,ostr.str(),2000);
		return -1;
	}

	cUser *other = mS.FindUserNick ( str );

	// check the nick
	if(!other)
	{
		ostr << "User " << str << "not found.";
		mS.DCPublicHS(ostr.str(),conn);
		return -2;
	}

	// check the privileges
	if(other->mClass >= conn->mpUser->mClass || other->mProtectFrom >= conn->mpUser->mClass)
	{
		ostr << "User " << str << "is too high for redir (or protected).";
		mS.DCPublicHS(ostr.str(),conn);
		return -3;
	}

	// create the message
	string omsg("$ForceMove ");
	omsg += msg->ChunkString(eCH_FM_DEST);
	other->mxConn->Send(omsg);
	omsg = "$To: ";
	omsg += msg->ChunkString(eCH_FM_NICK);
	omsg += " From: ";
	omsg += conn->mpUser->mNick;
	omsg += " $<";
	omsg += conn->mpUser->mNick;
	omsg += "> You are being re-directed to: " ;
	omsg += msg->ChunkString(eCH_FM_DEST);
	omsg += " because: ";
	omsg += msg->ChunkString(eCH_FM_REASON);
	
	if(other->mxConn)
	{
		// send it
		other->mxConn->Send(omsg);
		// close it
		other->mxConn->CloseNice(3000);
	}
	else
	{
		mS.DCPrivateHS("You can't move Hub-security.",conn);
	}
	return 0;
}


/** extended supports features */
int cDCProto::DCE_Supports(cMessageDC * msg, cConnDC * conn)
{
	string omsg("$Supports OpPlus");
	istringstream is;
	
	is.str(msg->mStr);
	string feature;
	is >> feature;
	while(1)
	{
		feature.clear();
		is >> feature;
		if(!feature.size()) break;
		if(feature == "OpPlus") conn->mFeatures |= eSF_OPPLUS;
	}
	conn->Send(omsg);
	return 0;
}

/** Network info (neo Modus) */
int cDCProto::DCM_NetInfo(cMessageDC * msg, cConnDC * )
{
	if(msg->SplitChunks()) return -1;
	
	return 0;
}

/** operator ban */
int cDCProto::DCO_TempBan(cMessageDC * msg, cConnDC * conn)
{
	if(!conn || !conn->mpUser || !conn->mpUser->mInList || conn->mpUser->mClass < eUC_OPERATOR) return -1;
	if(msg->SplitChunks()) return -1;

	ostringstream os;
	time_t period = 0;
	// calculate time
	if(msg->ChunkString(eCH_NB_TIME).size())
	{
	#warning todo DCO_TempBan still to test and fix bugs
		mS.Str2Period(msg->ChunkString(eCH_NB_TIME),os);
		if(!period)
		{
			mS.DCPublicHS(os.str(),conn);
			return -1;
		}
	}

	cUser *other = mS.FindUserNick(msg->ChunkString(eCH_NB_NICK));
	if(!other)
	{
		os << "User " << msg->ChunkString(eCH_NB_NICK) << " not found.";
		mS.DCPublicHS(os.str(),conn);
		return -1;
	}
	
	if(msg->mType == eDCO_TBAN  && !msg->ChunkString(eCH_NB_REASON).size())
	{
		os << "I don't ban without a reason!";
		mS.DCPublicHS(os.str(),conn);
		return -1;
	}

	if(other->mClass >= conn->mpUser->mClass || other->mProtectFrom >= conn->mpUser->mClass)
	{
		os << "You can't ban your superior or user is protected";
		mS.DCPublicHS(os.str(),conn);
		return -1;
	}

	if(!other->mxConn)
	{
		os << "You can't ban a special user: " << msg->ChunkString(eCH_NB_NICK);
		mS.DCPublicHS(os.str(),conn);
		return -1;
	}

	if(period)
		os << "You are being temporary banned for: " << msg->ChunkString(eCH_NB_TIME);
	else
		os << "You are banned";
	os << " because: " << msg->ChunkString(eCH_NB_REASON);

	mS.DCPrivateHS(os.str(), other->mxConn, &conn->mpUser->mNick);
	os.str(mS.mEmpty);

	int added = mS.mBanList.AddBan(
		other->mxConn->AddrIP(),
		msg->ChunkString(eCH_NB_NICK),
		mS.mEmpty, conn, period,
		msg->ChunkString(eCH_NB_REASON),
		eBT_NICKIP);
		
	if(added)
	{
		os << "Banned " << msg->ChunkString(eCH_NB_NICK);
		if(period) os << " for " << msg->ChunkString(eCH_NB_TIME) << endl;
		else os << " permanently " << endl;
		os << " Has IP: " << other->mxConn->AddrIP() << endl;
		if(other->mxConn->AddrHost().size())
			os << " Has Host: " << other->mxConn->AddrHost() << endl;
		os << " Banned by: ";
		if( added & eBT_IP   ) os << "ip ";
		if( added & eBT_NICK ) os << "nick ";
		if( added & eBT_HOST ) os << "host ";
		os << ".";
		mS.DCPublicHS(os.str(),conn);
		other->mxConn->CloseNice(1000);
		return 0;
	}

	os << "Not Banned." << endl;
	mS.DCPublicHS(os.str(), conn);
	return -2;
}

/** sends userlist and oplist */
int cDCProto::NickList(cConnDC *conn)
{
	try
	{
		if(mS.mRemakeNickList)
		{
			mS.mNickListString = "$NickList ";
			mS.mNickListString += mS.mC.hub_security + "$$";
			mS.GetNickListPlus1(mS.mNickListString, eUC_NORMUSER, conn);
			//conn->Send(list);
			mS.mOpListString = "$OpList ";
			mS.mOpListString += mS.mC.hub_security + "$$";
			mS.GetNickListPlus1(mS.mOpListString, eUC_OPERATOR, conn);
			mS.mRemakeNickList = false;
			if(conn->Log(3)) conn->LogStream() << "Remaking Nicklist" << endl;
		}
		if(conn->Log(3)) conn->LogStream() << "Sending Nicklist" << endl;
		conn->Send(mS.mNickListString);
		conn->Send(mS.mOpListString);
	}catch(...)
	{
		if(conn->ErrLog(2)) conn->LogStream() << "exception in DC_GetNickList" << endl;
		conn->CloseNow();
		return -1;
	}
	return 0;
}

/** test if text is a console command and parse it by console eventually
return 1 if it was a command else return 0 */
int cDCProto::ParseForCommands(const string &text, cConnDC *conn)
{
	ostringstream omsg;
	// test op's commands
	if(conn->mpUser->mClass >=  eUC_OPERATOR && text[0] == '!')
	{
		if(!mS.mCo.OpCommand(text,conn))
		{
			omsg << "Unknown command '" << text << "'. Try !help";
			mS.DCPublicHS(omsg.str(),conn);
		}
		return 1;
	}


	// check user commands
	if(text[0] == '+')
	{
		if(!mS.mCo.UsrCommand(text,conn))
		{
			omsg << "Unknown command '" << text << "'" << " try +help";
			mS.DCPublicHS(omsg.str(),conn);
		}
		return 1;
	}
	return 0;
}

/** operator unban */
int cDCProto::DCO_UnBan(cMessageDC * msg, cConnDC * conn)
{
	if(!conn || !conn->mpUser || !conn->mpUser->mInList || conn->mpUser->mClass < eUC_OPERATOR) return -1;
	if(msg->SplitChunks()) return -1;

	string ip, nick, host;
	ostringstream os;
//	if(msg->mType == eDCO_HOSTUNBAN ) host = msg->ChunkString(eCH_1_PAR);
//	if(msg->mType == eDCO_NICKUNBAN ) nick = msg->ChunkString(eCH_1_PAR);
	if(msg->mType == eDCO_UNBAN     ) ip   = msg->ChunkString(eCH_1_PARAM);

	int n = mS.mBanList.RemoveBan(ip,nick, host, eBT_NICKIP);

	if(n <= 0)
	{
		os << "Not found " << msg->ChunkString(eCH_1_PARAM) << " in banlist.";
		mS.DCPublicHS(os.str().data(),conn);
		return -1;
	}
	os << "Removed " << msg->ChunkString(eCH_1_PARAM) << endl;
	mS.DCPublicHS(os.str().data(),conn);
	return 1;

	return 0;
}

/** operator getbanlist */
int cDCProto::DCO_GetBanList(cMessageDC * msg, cConnDC * conn)
{
	if(!conn || !conn->mpUser || !conn->mpUser->mInList || conn->mpUser->mClass < eUC_OPERATOR) return -1;
	mS.mBanList.GetBanList(conn);
	return 0;
}

/** Bot information */
int cDCProto::DCB_BotINFO(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	if(conn->Log(2)) conn->LogStream() << "Bot visit: " << msg->ChunkString(eCH_1_PARAM) << endl;
	ostringstream os;
	// $HubINFO <hub name>$<hub address:port>$<hub description>$<max users>$<min share in bytes>$<min slots>$<max hubs>$<hub type>$<hubowner login>|
	char S='$';
	os << "$HubINFO "
		<< mS.mC.hub_name << S
		<< mS.mC.hub_host << ':' << mS.mPort << S
		<< mS.mC.hub_desc << S
		<< mS.mC.max_users << S
		<< (long long )(1024*1024)*mS.mC.min_share << S
		<< 1 << S << 20 << S
		<< "VerliHub" << S
		<< mS.mC.hub_owner;
		
	string str = os.str();
	conn->Send(str);
	return 0;
}

/** who is with given ip */
int cDCProto::DCO_WhoIP(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	string nicklist("$UsersWithIp ");
	string sep("$$");
	nicklist += msg->ChunkString(eCH_1_PARAM);
	nicklist += "$";
	mS.WhoIP(msg->ChunkString(eCH_1_PARAM),nicklist, sep, true);
	conn->Send(nicklist);
	return 0;
}

/** operator getbanlist filtered by the parameter */
int cDCProto::DCO_Banned(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	
	return 0;
}


/** operator get hub topic */
int cDCProto::DCO_GetTopic(cMessageDC *, cConnDC * conn)
{
	string topic("$HubTopic ");
	topic += mS.mC.hub_desc;
	conn->Send(topic);
	return 0;
}

/** operator get hub topic */
int cDCProto::DCO_SetTopic(cMessageDC * msg, cConnDC * conn)
{
	if(msg->SplitChunks()) return -1;
	if(!conn->mpUser->mInList) return -2;
	// check rights
	if(!conn->mpUser || conn->mpUser->mClass < eUC_ADMIN) return  -1;
	
	string &str = msg->ChunkString(eCH_1_PARAM);
	mS.mC.hub_desc = str;

	ostringstream os;
	os << "Topis is set to: " << str;
	mS.DCPublicHS(os.str(), conn);
	return 0;
}

};
