/***************************************************************************
                          cmeanfrequency.h  -  description
                             -------------------
    begin                : Sun Aug 31 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMEANFREQUENCY_H
#define CMEANFREQUENCY_H

#include "ctime.h"

/**
  *@author Daniel Muller
  */

/**
 * Mesuers a frequency over a given period of incomming events
 * events are counted by type T, they are summed
 * the given interval is divined into several subintervals
 * to make better resolution
 * */

template<class T> class cMeanFrequency
{
	public:
	/** period length over which it is measured */
	cTime mOverPeriod;
	/** length of every part or resolution */
	cTime mPeriodPart;
	/** start and end of mesured time */
	cTime mStart, mEnd;
	/** start (maybe end) of the partly period where we are now */
	cTime mPart;
	/** resolution of mesure */
	int mResolution;
	/** counts of events in every part of pertiod/resolution */
	T mCounts[10];
	int mStartIdx;
	/** number of filled periods */
	int mNumFill;

	/************** conctruction */
	cMeanFrequency()
	{
		cTime now;
		SetPeriod(0.,1);
		Reset(now);
	}
	
	cMeanFrequency(const cTime &now)
	{
		SetPeriod(0.,1);
		Reset(now);
	};

	cMeanFrequency(const cTime &now, double per, int res):
		mOverPeriod(per),
		mPeriodPart(per/res),
		mResolution(res)
	{
		Reset(now);
	};

	/** insert/add */
	void Insert(const cTime &now, T data=1)
	{
		Adjust(now);
		mCounts[(mStartIdx+mNumFill) % mResolution] += data;
	};

	/** calculate count over all period */
	int CountAll(const cTime &now)
	{
		Adjust(now);
		T sum=0;
		int i,j=mStartIdx;
		for(i=0; i < mNumFill; i++)
		{
			sum += mCounts[j++];
			if(j >= mResolution) j = 0;
		}
		return sum;
	};

	/** adjust state to current time */
	void Adjust(const cTime &now)
	{
		// need adjustment
		if( mEnd < now )
		{
			cTime t(mEnd);
			t += mOverPeriod; // if last adjustment happend more thern period ago
			// in this case we can empty all
			if( t < now ) return Reset(now);
			// shift until it's ok
			while(mEnd < now) Shift();
		}
		else if( mNumFill < mResolution)
		{
			while( mPart < now )
			{
				mPart += mPeriodPart;
				mNumFill ++;
			}
		}
	};

	// left (remove first part period)
	void Shift()
	{
		mEnd   += mPeriodPart;
		mStart += mPeriodPart;
		mCounts[mStartIdx] = 0;
		if(mNumFill) mNumFill --;
		mStartIdx ++;
		mStartIdx %= mResolution;
	};

	void Reset(const cTime &now)
	{
		memset(&mCounts,0, sizeof(mCounts));
		mStart = now;
		mEnd =  mStart;
		mEnd += mOverPeriod;
		mNumFill = 0;
		mStartIdx = 0;
		mPart = mStart;
		mPart += mPeriodPart;
	}

	/** setup the opeverperiod and the resolution */
	void SetPeriod(double per, int res)
	{
		mOverPeriod = cTime(per);
		mResolution = res;
		mPeriodPart = mOverPeriod / res;
	};

};


#endif
