/***************************************************************************
                          cpoller.cpp  -  description
                             -------------------
    begin                : Wed Jul 23 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cpoller.h"
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>

cPoller::cPoller(int init_size)
{
	mFDs.reserve(init_size);
	mConns.reserve(init_size);
}

cPoller::~cPoller()
{
}

/** register given sock descriptor for given anction for select, you can append some data in the void pointer */
int cPoller::Register(int sd, int actions, cAsyncConn * data)
{
	if(!data || sd < 0) return 0;

	mMaxSock = max(sd, mMaxSock);
	if((unsigned)mMaxSock >= mFDs.size())
	{
		// grow it 20% larger than necessary, so we reduce the count of resizing
		mFDs.resize(int(mMaxSock * 1.2 + 1));
		mConns.resize(mFDs.size(),NULL);
	}
	
	mFDs[sd].events = actions & POL_STANDARD;
	mFDs[sd].fd = sd;
	mConns[sd] = data;
	int extra = actions - mFDs[sd].events;
	data->mExtraPoll = extra;
	return 0;
}

/** register given sock descriptor for given anction for select, you can append some data in the void pointer */
int cPoller::UnRegister(int sd)
{
	static int repetition_count =0;
	// controlls
	if(sd < 0) return -1;
	if(mFDs.empty()) return -1;
	int size=mFDs.size();
	if(sd >= size) return -1;

	mFDs[sd].reset();
	mConns[sd] = NULL;
	
	if(++repetition_count > int(0.2 * mMaxSock + 1))
	{
		repetition_count=0;
		// update mMaxSock
		int i;
		for(i = mMaxSock; i > 0 && !mConns[i]; i--);
		if( i <= int(0.8*mMaxSock - 1) )
		{
			mFDs.resize(int(i*1.2+1));
			mConns.resize(mFDs.size(),NULL);
		}
		mMaxSock=i+1;
	}
	return 0;
}

/** select registered descriptors and return the number of events */
int cPoller::poll(int wp_sec)
{
	int ret, n=0;
	int total_size = mFDs.size(), size;
	int size_done = 0;

	while(total_size)
	{
		size = min( 1024 , total_size );
		TEMP_FAILURE_RETRY(ret = ::poll(&mFDs[0]+size_done, size, wp_sec));
		if(ret < 0)
		{
			if(errno != EAGAIN && errno != EINTR)
			{
				cout << "Poll error: " << errno << " size is " << size << " max is " << getdtablesize() << " wpsec is " << wp_sec << endl;
//				for(int i=0; i < size; i++)
//				{
//					cout << "sock " <<  (&mFDs[0]+i)->fd << "  ev:";
//					cout << (&mFDs[0]+i)->events << "  rev:";
//					cout << (&mFDs[0]+i)->revents << "  ";
//					cout << endl;
//				}
				exit(1);
			}
			return 0;
		}
		n+= ret;
		size_done+=size;
		total_size-=size;
	}
	return n;
}

cPoller::iterator & cPoller::iterator::operator++()
{
	// skips the unregistered, and those without any action
	if(mIt != mP->mFDs.end()) ++mIt;
	return SkipNulls();
};

