/***************************************************************************
                          cpoller.h  -  description
                             -------------------
    begin                : Wed Jul 23 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CPOLLER_H
#define CPOLLER_H

#include <cselector.h>
#include <vector>
#include <sys/poll.h>

using namespace std;

enum
{
 POL_ACCEPT   = POLLIN | POLLPRI, /* There is data to read */ /* There is urgent data to read */
 POL_INPUT    = POL_ACCEPT,
 POL_OUTPUT   = POLLOUT, /* Writing now will not block */
 POL_ERROR    = POLLERR | POLLHUP | POLLNVAL , /* Error condition *//* Hung up *//* Invalid request: fd not open */
 POL_STANDARD = POL_INPUT | POL_OUTPUT | POL_ERROR,
 POL_CLOSE    = POLLNVAL << 1, // higher then everything else in STANDARD
 POL_DEFAULT  = 0
};

/**poller is faster
  *@author Daniel Muller
  */

class cPoller : public cSelector  {
protected:
	// poll file descriptors
	struct cPollfd: public pollfd
	{
		cPollfd(){reset();};
		void reset(){fd=-1;events=revents=0;};
	};
	typedef vector<cPollfd> tFDs;
	typedef vector<cAsyncConn*> tConns;
	tFDs mFDs;
	tConns mConns;
	
public: 
	cPoller(int);
	virtual ~cPoller();
	
	/** the iterator */
	friend class iterator;
	class iterator
	{
		friend class cPoller;
		tFDs::iterator mIt;
		cPoller *mP;
		public:
		iterator():mP(NULL){};
		iterator(const iterator &pi, cPoller *cp):mIt(pi.mIt),mP(cp){};
		iterator(const tFDs::iterator &vi, cPoller *cp):mIt(vi),mP(cp){};
		iterator & operator=(const iterator& pi){mIt=pi.mIt; mP=pi.mP; return *this;};
		int operator!=(const iterator& pi){return mIt!=pi.mIt;};
		iterator & operator++();
		private:
		iterator & SkipNulls()
		{
			if(!mP) return *this;
			//cAsyncConn **ci=&(mP->mConns[0];
			
			while(
				mIt != mP->mFDs.end() &&
				(
					mIt->fd <= 0 ||
					mP->mConns[mIt->fd] == NULL ||
					!(mIt->revents + mP->mConns[mIt->fd]->mExtraPoll)
				)
			)++mIt;
			return *this;
		};
		// end of class
	};

	/** select registered descriptors and return the number of events */
	int poll(int wp_sec);
	/** register given sock descriptor for given anction for select, you can append some data in the void pointer */
	int Register(int sd, int actions, cAsyncConn * data);
	/** register given sock descriptor for given anction for select, you can append some data in the void pointer */
	int UnRegister(int sd);
	/** return connection by the iterator */
	inline cAsyncConn * Conn(const iterator & it){return mConns[it.mIt->fd];}
	/** return revent by the iterator */
	inline int Actions(const iterator & it){return it.mIt->revents;}

	/** begin end end iterators */
	iterator begin(){return iterator(mFDs.begin(),this).SkipNulls();}
	iterator end()  {return iterator(mFDs.end()  ,this);}
};

#endif
