/***************************************************************************
                          creglist.cpp  -  description
                             -------------------
    begin                : Mon Jul 7 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "creglist.h"
#include "cserverdc.h"
#include "cuser.h"
#include "creguserinfo.h"
#include <sstream>
#include <cctype>
#include <algorithm>

namespace nDirectConnect
{


cRegList::cRegList(cServerDC &serv):mS(serv)
{
}
cRegList::~cRegList(){
}

/** creates empty table in database if not exists */
int cRegList::CreateTable()
{
	string query;
	query = "CREATE TABLE IF NOT EXISTS reglist (";
	query+= "nick varchar(60) NOT NULL PRIMARY KEY, "; // users's nick
	query+= "class int(2) default 1, "; // user's class
	query+= "reg_date datetime, "; // registration date
	query+= "reg_op varchar(60), "; // registration operator, the one who gave the registration
	query+= "pwd_change bool default 1, "; // user must change password
	query+= "pwd_crypt bool default 0, "; // crypt password
	query+= "login_pwd varchar(60), "; // password
	query+= "login_last datetime default 0, "; // last login
	query+= "login_count int(11) default 0, "; // succesfull login count
	query+= "login_ip varchar(16), "; // last ip
	query+= "error_last datetime, "; // unsuccessfull last login date
	query+= "error_cnt int(11) default 0, "; // unsuccessfull login count
	query+= "error_ip varchar(16), "; // ip on unsuccessfull login
	query+= "enabled bool default 1, "; // account enabled
	query+= "email varchar(60), "; // user's email
	query+= "man_ip varchar(16)"; // if user manages an ip or range, this is it
	query+= ")";
	
	if(mysql_query(&mS.db_handle, query.data()))
	{
		cout << query << endl;
		cout << "Mysql Error: " << mysql_error( &mS.db_handle ) << endl;
	}

//	query = "CREATE TABLE IF NOT EXISTS reg_demand (";
//	query+= "ip
	return 0;
}

/** find nick in reglist
if not foud return 0
else return 1 and fill in the reuserinfo parameter */
int cRegList::FindRegInfo(cRegUserInfo &ui, const string &nick)
{
	string query;
	MYSQL_RES *result;
	MYSQL_ROW row;

	// create query
	query = "SELECT nick, class, pwd_crypt, login_pwd, login_last,";
	query+= " login_ip, error_last, error_ip, pwd_change, enabled";
	query+= " FROM reglist WHERE LOWER(reglist.nick) = LOWER('";
	query+= nick;
	query+= "')";

	// call query
	if(mysql_query(&mS.db_handle, query.data()))
	{
		cout << query << endl;
		cout << "Mysql Error: " << mysql_error( &mS.db_handle ) << endl;
		return 0;
	}

	// take result
	result = mysql_store_result(&mS.db_handle);

	// count lines
	int row_count= mysql_num_rows(result);
  // user is not registered
	if(!row_count){
		mysql_free_result(result);
		return 0;
	}

	// fetch a row
	row = mysql_fetch_row(result);
	
	// fill in structure
	ui.mNick =      row[0];
	ui.mClass =     row[1]? atoi(row[1]) : 0;
	ui.mPWCrypt =   row[2]? atoi(row[2]) : 0;
	ui.mPasswd =    row[3]? row[3]       : "";
	ui.mLoginLast = row[4]? atol(row[4]) : 0;
	ui.mLoginIP =   row[5]? row[5]       : "";
	ui.mErrorLast = row[6]? atol(row[6]) : 0;
	ui.mErrorIP =   row[7]? row[7]       : "";
	ui.mPwdChange = row[8]? atoi(row[8]) : 0;
	ui.mEnabled   = row[9]? atoi(row[9]) : 0;
	
	mysql_free_result(result);
	return 1;
}

/** add registered user */
int cRegList::AddRegUser(const string &nick, cConnDC *op, int cl)
{
	ostringstream query;
	cRegUserInfo ui;

	if(FindRegInfo(ui, nick)) return 1;
	//if(!mS.FindUserNick(nick)) return 2;
	
	query << "INSERT INTO reglist ( nick, class, reg_date, reg_op ) VALUES (";
	query << "'" << nick << "', ";
	query << cl << ", ";
	query << "NOW(), ";
	query << "'" << op->mpUser->mNick << "'";
	query << ")";

	//cout << query.str() << endl;
	if(mysql_query(&mS.db_handle, query.str().data()))
	{
		cout << query << endl;
		cout << "Mysql Error: " << mysql_error( &mS.db_handle ) << endl;
		return 1;
	}
	return 0;
}


/** No descriptions */
int cRegList::ChangePwd(const string &nick, const string &pwd, int crypt)
{
	cRegUserInfo ui;

	//if(FindRegInfo(ui, nick)) return 1;
	ostringstream query;

	query << "UPDATE reglist SET login_pwd=";
	if(crypt) query << "CRYPT(";
	query << "'" << pwd << "'";
	if(crypt) query << ")";
	query << ", pwd_change=0";
	query << ", pwd_crypt=" << crypt;
	query << " WHERE nick='"  << nick << "'";

	if(mysql_query(&mS.db_handle, query.str().data()))
	{
		cout << query << endl;
		cout << "Mysql Error: " << mysql_error( &mS.db_handle ) << endl;
		return 1;
	}
	return 0;
}

/** No descriptions */
int cRegList::SetVar(const string &nick, string &field, string &value)
{
	ostringstream query;

	query << "UPDATE reglist SET " << field << "='" << value << "'";
	query << " WHERE nick='"  << nick << "'";

	if(mysql_query(&mS.db_handle, query.str().data()))
	{
		cout << query << endl;
		cout << "Mysql Error: " << mysql_error( &mS.db_handle ) << endl;
		return 0;
	}
	return 1;
}

/** log that user logged in */
int cRegList::Login(cConnDC *conn, const string &nick)
{
	ostringstream query;
	query << "UPDATE reglist SET login_last = NOW(), login_count=login_count+1, login_ip ='"
		<< conn->AddrIP() << "' WHERE nick ='" << nick << "'";
	if(mysql_query(&mS.db_handle, query.str().data()))
	{
		cout << query << endl;
		cout << "Mysql Error: " << mysql_error( &mS.db_handle ) << endl;
		return 1;
	}
	return 0;
}

/** log that user logged in with error*/
int cRegList::LoginError(cConnDC *conn, const string &nick)
{
	ostringstream query;
	query << "UPDATE reglist SET error_last = NOW(), error_cnt=error_cnt+1, error_ip ='"
		<< conn->AddrIP() << "' WHERE nick ='" << nick << "'";
	if(mysql_query(&mS.db_handle, query.str().data()))
	{
		cout << query.str() << endl;
		cout << "Mysql Error: " << mysql_error( &mS.db_handle ) << endl;
		return 1;
	}
	return 0;
}

/**

How registering works:
----------------------

1/ user <Nick> asks to be registered
2/ if operator is ok with it, he uses command !regnewuser <Nick> [<class>=1] to prepare an account for the user..
	* this checks if such user is registered (if so error occurs)
	* creates default reguser data into the reglist, which is marked as pwd_change
	* then it sends pm to the user asking him por passwrd and some other settings
3/ user provides password with +passwd <new_pass> [<crypt>=0], which is enabled only if pwd_change is marked
	* if user doesn't provide any password he won't be registered
	* if password is provided and is ok, user finishes his registration
4/ to change password user asks any op again, op uses command !regpasswd <Nick> to enable password changing
5/ to change user's class op uses commands !regclass <Nick> <new_class> ; new_class must be less then op's class
6/ to ungegiser use op command !regdeluser <Nick> - this disables the registration by setting a flag
7/ to purge a disabled record, admin can do !regpurge <Nick>

Note: registered users aren't banned by ip only, but by their nick too
*/

};
