/***************************************************************************
                          cselelector.cpp  -  description
                             -------------------
    begin                : Tue Jun 10 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cselector.h"
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

cSelector::cSelector():mDataList(0),mMaxSock(0)
{
}

cSelector::~cSelector()
{
}

/** preapres selection masks with all registered sockets*/
int cSelector::CreateMasksSelect(fd_set *r_set,fd_set *w_set,fd_set *e_set)
{
	memset(r_set,0,sizeof(fd_set));
	memset(w_set,0,sizeof(fd_set));
	memset(e_set,0,sizeof(fd_set));

	//** fill bitmaps in
	int i;
	for(i = 0; i <= mMaxSock; i++)
	{
		if(! mDataList[i].conn) continue;
		else
		{
			int act=mDataList[i].activity;
			if( act & SEL_READ  )FD_SET(i,r_set);
			if( act & SEL_WRITE )FD_SET(i,w_set);
			if( act & SEL_ERROR )FD_SET(i,e_set);
		}
	}
	return 0;
}

/** uses selection masks to create the list of selected entires */
int cSelector::InterpretMasksSelect(fd_set * r_set, fd_set * w_set, fd_set * e_set)
{
	int extra=0;
	tsData key;
	int i;
	
	for(i = 0; i <= mMaxSock; i++)
	{
		if(!mDataList[i].conn) continue;
		else
		{
			int act,act0;
			key = mDataList[i];
			act0 = key.activity;
			key.activity=act0 & SEL_CLOSE;
			
			if( (act=(act0 & SEL_READ  )) && FD_ISSET(i,r_set)) key.activity |= act;
			if( (act=(act0 & SEL_WRITE )) && FD_ISSET(i,w_set)) key.activity |= act;
			if( (act=(act0 & SEL_ERROR )) && FD_ISSET(i,e_set)) key.activity |= act;
			if(key.activity) mKeyList.push_back(key);
			if(key.activity == SEL_CLOSE) extra++; // if there is not another activity
		}
	}
	return extra;
}

/** select registered descriptors and return the number of events */
int cSelector::select(int wp_sec)
{
	fd_set r_set, w_set, e_set;
	timeval tv;
	memset(&tv, 0, sizeof(struct timeval));
	tv.tv_sec  = wp_sec;
	tv.tv_usec = 0;
	
	// clear result list
	mKeyList.clear();

	int ret=0, extra=0;
	extra += CreateMasksSelect(&r_set,&w_set,&e_set);

	//** perform system select 
	TEMP_FAILURE_RETRY( ret = ::select (mMaxSock+1, &r_set, &w_set, &e_set, (wp_sec == -1) ? 0: &tv) );
	if (ret == -1)
	{
		string err = "cSelector::select error: ";
		err += strerror(errno);
		err += mMaxSock;
		throw err.data();
	}

	//** create list of selected keys 
	//if(!ret) return 0; // not this, because there may be some data in buffer
	extra += InterpretMasksSelect(&r_set,&w_set,&e_set);
	
	return ret+extra;
}

/** register given sock descriptor for given anction for select, you can append some data in the void pointer */
int cSelector::Register(int sd, int actions, cAsyncConn *data)
{
	mMaxSock = max(sd, mMaxSock);
  // resize vector if necessary
	if((unsigned)mMaxSock >= mDataList.size())
		// grow it 20% larger than necessary, so we reduce the count of resizing
		mDataList.resize(int(mMaxSock * 1.2 + 1));
	
	mDataList[sd] = cSelector::tsData(actions, data);
	return 0;
}

int cSelector::UnRegister(int sd)
{
	static int repetition_count =0;
	if(sd <= 0)
		return -1;
	if(mDataList.empty())
		return -1;
	int size=mDataList.size();
	if(sd >= size )
		return -1;
	mDataList[sd] = cSelector::tsData(); // nulify it
	if(++repetition_count > int(0.2*mMaxSock + 1))
	{
		repetition_count=0;
		// update mMaxSock
		int i;
		for(i = mMaxSock; i > 0 && !mDataList[i].conn; i--);
		if( i <= int(0.8*mMaxSock - 1) )
			mDataList.resize(int(mMaxSock*1.2+1));
		mMaxSock=i+1;
	}
	return 0;
}

/** sleep */
void cSelector::sleep(unsigned int usec)
{
	::usleep(usec);
}

/** select registered descriptors and return the number of events */
int cSelector::poll(int wp_sec)
{
	int count = mDataList.size();
	int closen;
	pollfd *polls = new pollfd[count];
	if(!polls){throw " cSelector::poll no memory";}

	// clear result list
	mKeyList.clear();

	//** fill structs in
	count=CreateMasksPoll(polls,closen);
	
	int ret;
	TEMP_FAILURE_RETRY(ret = ::poll(polls, count, wp_sec));

	// reserve some memory in advance
	mKeyList.reserve(ret+closen+1);

	if(ret < 0)
	{
		delete polls;
		return 0;
	}

	ret += InterpretMasksPoll();
	delete polls;
	return ret;
}

/** preapres selection masks for poll */
int cSelector::CreateMasksPoll(pollfd* polls, int &closen)
{
	int j,i=0;
	closen=0;
	for(j = 0; j <= mMaxSock; j++)
	{
		if(! mDataList[j].conn || !mDataList[j].activity) continue;
		else
		{
			polls[i].fd= j;
			polls[i].events=0;
			if( mDataList[j].activity & SEL_READ ) polls[i].events |= (POLLIN | POLLPRI);
			if( mDataList[j].activity & SEL_WRITE) polls[i].events |= POLLOUT;
			if( mDataList[j].activity & SEL_CLOSE) closen++;
			mDataList[j].polld=&(polls[i]);
			i++;
		}
	}
	return i;
}

/** uses selection masks to create the list of selected entires */
int cSelector::InterpretMasksPoll()
{
	int i,ret=0;
	int act,act0;
	pollfd *curr;
	tsData key, *data;
	key.polld=0;
	
	for(i = 0; i <= mMaxSock; i++)
	{
		data = & mDataList[i];
		curr = data->polld;
		if(!data->conn || !curr) continue;
		else
		{
			act0 = data->activity;
			key.activity = act0 & SEL_CLOSE;
			key.conn = data->conn;
			if( (act=(act0 & SEL_READ  )) && curr->revents & (POLLIN |POLLPRI) ) key.activity |= act;
			if( (act=(act0 & SEL_WRITE )) && curr->revents & POLLOUT           ) key.activity |= act;
			if( (act=(act0 & SEL_ERROR )) && curr->revents & (POLLERR|POLLHUP|POLLNVAL)) key.activity |= act;
			if(key.activity) mKeyList.push_back(key);
			if(key.activity == SEL_CLOSE) ret++; // if there is not another activity
		}
	}
	return ret;
}
