/***************************************************************************
                          cselector.h  -  description
                             -------------------
    begin                : Tue Jun 10 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSELECTOR_H
#define CSELECTOR_H

/**socket selector class
  *@author Daniel Muller
  */

class cAsyncConn;

#include <vector>
#include <list>
#include <sys/poll.h>
#include "casyncconn.h"
#ifndef TEMP_FAILURE_RETRY
#define TEMP_FAILURE_RETRY(expression) \
  (__extension__                    \
    ({ long int __result;                 \
       do __result = (long int) (expression);             \
       while (__result == -1L && errno == EINTR);           \
       __result; }))

#endif

using namespace std;

class cAsyncConn;

enum
{
	SEL_ACCEPT = 1 << 0, // listen connection ready for accepting
	SEL_INPUT  = 1 << 1, // incoming data ready, or ptiority data ready
	SEL_OUTPUT = 1 << 2, // ready for aoutput
	SEL_ERROR  = 1 << 3, // error occured
	SEL_CLOSE  = 1 << 4, // connection is to be closed by us
	SEL_READ    = SEL_INPUT+SEL_ACCEPT, // reading events
	SEL_WRITE   = SEL_OUTPUT,// writing events
	SEL_DEFAULT = SEL_ERROR // default events to watch
};

class cSelector {
public: 
	cSelector();
	virtual ~cSelector();
	/** select registered descriptors and return the number of events */
	int select(int wp_sec=-1);
	/** select registered descriptors and return the number of events */
	virtual int poll  (int wp_sec=-1);
	/** register given sock descriptor for given anction for select, you can append some data in the void pointer */
	virtual int Register(int sd, int actions, cAsyncConn *data=NULL);
	virtual int UnRegister(int sd);
	
	/** data list */
	struct tsData{
		cAsyncConn *conn;
		int activity;
		pollfd *polld;
		tsData(int a=0, cAsyncConn *c=NULL, pollfd *p=NULL):conn(c),activity(a),polld(p){}
	};
	typedef vector<tsData> tDataList;
	typedef tDataList::iterator tDLIt;
	tDataList mDataList;

	/** key list */
	typedef vector<tsData> tKeyList;
	typedef tKeyList::iterator tKLIt;
	tKeyList mKeyList;
	/** the maximal socket desriptor */
	int mMaxSock;
	
	/** Iterations */
	typedef tKLIt iterator;
	iterator begin(){return mKeyList.begin();}
	iterator end(){return mKeyList.end();};
  /** sleep */
  void sleep(unsigned int msec);

private: // Private methods
  /** preapres selection masks */
  int CreateMasksSelect(fd_set *r_set,fd_set *w_set,fd_set *e_set);
  /** uses selection masks to create the list of selected entires */
  int InterpretMasksSelect(fd_set * r_set, fd_set * w_set, fd_set * e_set);
  /** preapres selection masks for poll */
  int CreateMasksPoll(pollfd*,int &);
  /** uses selection masks to create the list of polled entires */
  int InterpretMasksPoll();
};

#endif

