/***************************************************************************
                          cserverdc.cpp  -  description
                             -------------------
    begin                : Fri May 9 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cserverdc.h"
#include "cinterpolexp.h"
#include "cconndc.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdlib.h> /* for strtoll */
#include <stdio.h> /* for remove and rename */
#include <fstream>
#include <cctype>
#include <algorithm>

#define HUB_VERSION 0
#define HUB_VERSION_MIN 9.4
#define HUB_VERSION_CLASS "alpha"
#define HUB_VERSION_STRING "0.9.4alpha"

using namespace std;
using namespace nUtils;
namespace nDirectConnect
{

hash<const char *> cServerDC::hHashStr::msHash;

cServerDC::cServerDC(int p):
	cAsyncSocketServer(p),
	mP(*this),
	mCo(*this),
	mC("./.verlihub/config",*this),
	mR(*this),
	mSysLoad(eSL_COOL),
	mUserCount(0),
	mTotalShare(0),
	mBanList(*this, ".verlihub/banlist")
{
	SetClassName("cServerDC");
	mC.Load();
	//CfgLoadReglist();
	mStartTime = ::time(NULL);

	// connect to mysql database
	if(!mMySQL.Connect(mC.db_host, mC.db_user, mC.db_pass, mC.db_data))
		throw "Can't connect to mysql database :(.";

	memset(&db_handle,0, sizeof(db_handle));
	if(!mysql_init(&db_handle))
		throw "Can't connect to mysql database :(.";
	mysql_options(&db_handle,MYSQL_OPT_COMPRESS,0);
	if(!mysql_real_connect(&db_handle, mC.db_host.data(), mC.db_user.data(), mC.db_pass.data(), mC.db_data.data(),0,0,0))
		throw "Can't connect to mysql";
	mR.CreateTable();
	mBanList.CreateTable();
	mRemakeNickList = true;
	// add the user
	cUser *VerliHub=new cUser(mC.hub_security);
	VerliHub->mClass=tUserCl(10);
	VerliHub->mMyINFO="VerliHub$ $Hub\x9$$0$";
	AddToList(VerliHub);
	mNetOutLog.open("./net_out.log",ios::out);
}

cServerDC::~cServerDC()
{
	if(Log(1)) LogStream() << "Destructor cServerDC" << endl;
	mNetOutLog.close();
	// destruct the lists of pointers
	for (tTFIt i = mTmpFunc.begin(); i != mTmpFunc.end(); i++ ) if(*i) delete *i;
}

/**
 * The rules for allowed messages
 * ------------------------------
 *  *** USER IS NOT LOGGED IN USERLIST ***
 *  . mask = KEY | VALIDATENICK | MYPASS | VERSION | GETNICKLIST | MYINFO
 *  . (they may apprear in given order)
 *  . GETNICKLIST only if eLS_KEYOK && eLS_VALNICK && eLS_PASSWD && eLS_VERSION
 *  *** USER IS IN USERLIST ***
 *  . EVERYTHING except KEY VALIDATENICK MYPASS VERSION
 *  -------
 *  *** hub is loaded ***
 *	SOME MAY BE REMOVED: SEARCH*, SR, MYINFO_ALL , CHAT, GETNICKLIST
 *	  
 **/

tMsgAct cServerDC::Filter( tDCMsg msg, cConnDC *conn )
{
	tMsgAct result = eMA_PROCEED;
	if(!conn) // this is strange
	{
		if(ErrLog(0)) LogStream() << "Got NULL conn into filter" << endl;
		return eMA_ERROR;
	}
	if(!conn->mpUser || !conn->mpUser->mInList)
	{
		switch(msg)
		{
			case eDC_KEY:
			case eDC_VALIDATENICK:
			case eDC_MYPASS:
			case eDC_VERSION:
			case eDC_GETNICKLIST:
			case eDC_MYNIFO:
			case eDC_UNKNOWN:
			break;
			default: result = eMA_HANGUP;
		}
	}
	else
	{
		switch(msg)
		{
			case eDC_KEY:
			case eDC_VALIDATENICK:
			case eDC_MYPASS:
			case eDC_VERSION:
				result=eMA_HANGUP;
			break;
			default: break;
		}
	}

	switch( mSysLoad )
	{
		case eSL_SYSTEM_DOWN: return eMA_TBAN; break; // ignore and pray
		case eSL_CRITICAL: return eMA_HANGUP1; break;
		case eSL_SQEEZY: break;
		case eSL_HURRY: break;
		case eSL_COOL: break;
		default: break;
	}
	return result; 
}


int cServerDC::TreatMsg( cMessageDC *msg, cConnDC *conn)
{
/**
 * How to treat messages ?
 * -----------------------
 *  1. parse the message
 *  2. determine the set of available messages
 *  3. if message isn't in the set (ignore or disconnect or even ban ;)
 *  4. filter messaes depending on load, frequency etc... (in function of user, * conn, message, and other)
 *  5. if user not logged yet do the logging procedure ....
 *  6. discard the message
 *
 * some actions may be postponned (limited time)
 * */

/**
 * The loging procedure
 * --------------------
 *  . get the key, nick
 *  . dependig on hub niok, cacity and load, accept connection or not
 *  . check the password (false => disconnect, tban for few seconds )
 *  . when all necessary things are filled in, proceed to user list 
 *  . if op .... you know what
 **/

/**
 * Filter may result in some actions
 * ---------------------------------
 *  PROCEED // proceed in full kind of way
 *  LIMITED // do only a part of the thing
 *  LATER		// try to do it later or not
 *  IGNORE	// don't do it anyway
 *  HANGUP	// close connection (with a message)
 *  HANGUP1 // close connection immediately
 *  TBAN		// close conn and temp ban nick and ip
 **/

	tMsgAct action = Filter(msg->mType,conn);

	if((msg->mLen > mC.max_length[msg->mType]) && (action < eMA_IGNORE ))
		action = eMA_IGNORE;
	
	switch ( msg->mType )
	{
		case eDC_UNKNOWN: break;
		case eDC_UNPARSED: msg->Parse(); return TreatMsg(msg, conn); break;
		case eDC_KEY: mP.DC_Key(msg, conn); break;
		case eDC_VALIDATENICK: mP.DC_ValidateNick(msg, conn); break;
		case eDC_MYPASS: mP.DC_MyPass( msg, conn); break;
		case eDC_VERSION: mP.DC_Version( msg, conn); break;
		case eDC_GETNICKLIST: mP.DC_GetNickList( msg, conn ); break;
		case eDC_MYNIFO: mP.DC_MyINFO( msg, conn ); break;
		case eDC_GETINFO: mP.DC_GetINFO( msg, conn ); break;
		case eDC_CONNECTTOME: mP.DC_ConnectToMe( msg, conn); break;
		case eDC_MCONNECTTOME: mP.DC_MultiConnectToMe( msg, conn);break;
		case eDC_RCONNECTTOME: mP.DC_RevConnectToMe (msg, conn); break;
		case eDC_TO: mP.DC_To ( msg, conn ); break;
		case eDC_CHAT: mP.DC_Chat (msg, conn ); break;
		case eDC_OPFORCEMOVE: mP.DC_OpForceMove( msg, conn); break;
		case eDC_KICK: mP.DC_Kick( msg, conn); break;
		case eDC_SEARCH:
		case eDC_SEARCH_PAS: 
		case eDC_MSEARCH:
		case eDC_MSEARCH_PAS:mP.DC_Search( msg, conn); break;
		case eDC_SR: mP.DC_SR ( msg, conn ); break;
		case eDC_QUIT: DCPublicHS("Bye Bye",conn); conn->CloseNice(2000); break;
		case eDCE_SUPPORTS: mP.DCE_Supports( msg, conn); break;
		case eDCO_BAN:
		case eDCO_TBAN: mP.DCO_TempBan(msg, conn); break;
		case eDCO_UNBAN: mP.DCO_UnBan(msg, conn); break;
		case eDCO_GETBANLIST: mP.DCO_GetBanList(msg, conn); break;
		case eDCO_WHOIP: mP.DCO_WhoIP(msg, conn); break;
		case eDCO_BANNED: mP.DCO_Banned(msg, conn); break;
		case eDCO_GETTOPIC: mP.DCO_GetTopic(msg, conn); break;
		case eDCO_SETTOPIC: mP.DCO_SetTopic(msg, conn); break;
		case eDCB_BOTINFO: mP.DCB_BotINFO(msg, conn); break;
		default: if(Log(1)) LogStream() << "Incoming untreated event" << endl; break;
	}
	
	//delete msg;
	//msg = NULL;
	return 1;
}

/** sends a public chat message to a given connection */
int cServerDC::DCPublic(const string &from, const string &txt, cConnDC *conn)
{
	static string msg;
	msg.erase();
	msg = "<";
	msg += from + "> " + txt ;
	conn->Send( msg );
	return 1;
}

/** send a hub security public message */
int cServerDC::DCPublicHS(const string &text, cConnDC *conn)
{
	return DCPublic( mC.hub_security, text,  conn );
}

/** send a hub security private message */
int cServerDC::DCPrivateHS(const string & text, cConnDC * conn, string *from)
{
	// $To: <othernick> From: <nick> $<<nick>> <message>
	static string msg;
	msg = "$To: ";
	msg+= conn->mpUser->mNick;
	msg+= " From: ";
	msg+= mC.hub_security;
	msg+= " $<";
	if(from)
		msg+= *from;
	else
		msg+= mC.hub_security;
	msg+= "> ";
	msg+= text;
	return conn->Send(msg);
}

/** adds a user into a user list */
bool cServerDC::AddToList(cUser *usr)
{
	if(!usr)
	{
		if(ErrLog(1)) LogStream() << "Adding a NULL user to userlist" << endl;
		return false;
	}
	if(usr->mInList)
	{
		if(ErrLog(2)) LogStream() << "User is already in the user list, he says it " << endl;
		return false;
	}
	std::string lnick(usr->mNick);
//	std::transform(lnick.begin(), lnick.end(), lnick.begin(), ::tolower);
	
	cUser *other = FindUserNick( lnick );
	if(other)
	{
		if(ErrLog(2)) LogStream() << "Adding twise user with same hash " << usr->mNick << " against "<< other->mNick << endl;
		return false;
	}
	
	mUserList.insert(tNickUserPair(lnick,usr));
	if(usr->mxConn && usr->mxConn->Log(3)) usr->mxConn->LogStream() << "Adding at the end of Nicklist" << endl;
	if(mNickListString[mNickListString.size()-1]=='|')
		mNickListString.resize(mNickListString.size()-1);
	mNickListString+=usr->mNick;
	mNickListString+="$$";
	if(usr->mClass >= eUC_OPERATOR)
	{
		if(mOpListString[mOpListString.size()-1]=='|')
			mOpListString.resize(mOpListString.size()-1);
		mOpListString+=usr->mNick;
		mOpListString+="$$";
	}
	return true;
}

/** find user with given nick, if not there return NULL */
cUser * cServerDC::FindUserNick(const string &nick)
{
	// make nick lowercase
	string lnick(nick);
//	std::transform(lnick.begin(), lnick.end(), lnick.begin(), ::tolower);
	// find it
	tULIt it= mUserList.find(lnick);
	if(it ==  mUserList.end())
		return NULL;
	else
		return it->second;
}

/** removes user with given nick from uselist, return true on success */
bool cServerDC::RemoveNick(const string &nick)
{
	string lnick(nick);
//	std::transform(lnick.begin(), lnick.end(), lnick.begin(), ::tolower);
	mUserList[lnick] = NULL;
	mUserList.erase(lnick);
	static string omsg;
	omsg="$Quit ";
	omsg+=nick;
	// delayed myinfo implies delay of quit too, otherwise there would be mess in peoples userslists
	if(!mC.delayed_myinfo)
		SendToAll(omsg, eUC_NORMUSER, eUC_MASTER);
	else
		mSendAllBuf+=omsg+"|";

	mRemakeNickList = true;
	return true;
}

/** returns the nicklist i format nick$$... */
void cServerDC::GetNickList(string &list, int cls)
{
	// reserve memory for string
	list.reserve(list.size() + mUserCount * (mC.max_nick+2));
	cConnDC *conn;
	cUser *usr;

	// the has_map iterator is too slow, so I use this list iterator, much faster
	for(tCLIt i=mConnList.begin(); i!= mConnList.end(); i++)
	{
		conn = (cConnDC*)(*i);
		if(
				conn->ok &&
				conn->mpUser
			)
			usr= conn->mpUser;
		else continue;
		if(
			usr->mInList &&
			usr->mClass >= cls
		)
		{
			list += usr->mNick;
			list += "$$";
		}
	}
	//list+= '|'; // there are two pipes at the end, Send will add the other
}

/** send the same data to every user */
void cServerDC::SendToAll(string &data, int cm,int cM)
{
	cConnDC *conn;
	tCLIt i;
	int size = data.size();
	
	// prepare data
	if(size >= MAX_SEND_SIZE-1)
	{
		if(Log(2))
			LogStream() << "Truncating too long message from: "
				<< data.size() << " to " << MAX_SEND_SIZE -1 << " Message starts with: " << data.substr(0,10) << endl;
		data.resize( MAX_SEND_SIZE -1,' ');
		size = MAX_SEND_SIZE -1;
	}
	if(data[data.size()-1] !='|')
	{
		data.append("|");
		size ++;
	}

	int count=0;
	for(i=mConnList.begin(); i!= mConnList.end(); i++)
	{
		conn=(cConnDC *)(*i);
		if(conn && conn->ok && conn->mWritable && conn->mpUser && conn->mpUser->mInList)
		{
			conn->Write(data);
			++count;
		}
	}
	if(Log(5)) LogStream() << "ALL << " << data.substr(0,100) << endl;
	if(msLogLevel >= 3)
		mNetOutLog << ((long long)count) *data.size() << " "
			<< data.size() << " "
			<< count << " " << data.substr(0,10) << endl;
}

/** sends to every user data composed by start+nick+end */
void cServerDC::SendToAllWithNick(const string &start,const string &end, int cm,int cM)
{
	static string str;
	cConnDC *conn;
	tCLIt i;
	for(i=mConnList.begin(); i!= mConnList.end(); i++)
	{
		conn=(cConnDC *)(*i);
		if(
			conn &&
			conn->ok &&
			conn->mpUser &&
			conn->mpUser->mInList &&
			conn->mpUser->mClass >= cm &&
			conn->mpUser->mClass <= cM
			)
		{
			str=start + conn->mpUser->mNick + end;
			conn->Send(str);
		}
	}
}

/** return negative if conn should be removed */
int cServerDC::OnNewConn(cAsyncConn *nc)
{
	cConnDC *conn = (cConnDC *)nc;
	stringstream errmsg,os;
	if(!conn) return -1;
	string omsg;
	
	omsg="$Lock EXTENDEDPROTOCOL_verlihub Pk=version0.9.3";
	conn->Send(omsg);
	os << "This Hub is running version " << HUB_VERSION << "." << HUB_VERSION_MIN
		<< " " << HUB_VERSION_CLASS << " of VerliHub (RunTime:";
	os << ((time(NULL) - mStartTime)  ) / (24*3600) << "days ";
	os << ((time(NULL) - mStartTime) % (24*3600)) / 3600 << "h ";
	os << ((time(NULL) - mStartTime) % 3600) / 60 << "min";
	os << ").";
	DCPublicHS(os.str(), conn);
		
	return 0;
}

/** create a connection object to the sock descriptor */
cAsyncConn * cServerDC::FactoryConnection(int sd)
{
	cConnDC *conn=new cConnDC(sd, this);
	return (cAsyncConn*) conn;
}

/** create somehow a string to get line for given connection, ad return th pointer */
string * cServerDC::FactoryString(cAsyncConn *conn)
{
	cConnDC *c = (cConnDC*) conn;
	return c->FactoryString();
}

/** treat message for given connection */
void cServerDC::OnNewMessage(cAsyncConn *conn, string *str)
{
	cConnDC* c=(cConnDC*)conn;
	if(conn->Log(4)) conn->LogStream() << "IN: " << (*str) << "|" << endl;
	c->mMsg.Parse();
	TreatMsg(&(c->mMsg), c);  // server treats and discards message
}

/** loads te list of registered users */
int cServerDC::CfgLoadReglist()
{
	ifstream is("./.verlihub/reglist");
	if(!is) return 1;
	cRegUserInfo *ui;
	while(!is.eof())
	{
		ui = new cRegUserInfo;
		if(!ui) throw "Can't create new reguser" ;
		is >> (*ui);
		if(is.eof()){
			delete ui;
			break;
		}
		mRegUsers.insert(tRegUsrPair(ui->GetNick(),ui));
	}
	is.close();
	return 0;
}

/** is called when user is about to get into the userslist */
void cServerDC::DoUserLogin(cConnDC *conn)
{
	if(eLS_LOGIN_DONE != conn->GetLSFlag(eLS_LOGIN_DONE))
	{
		if(conn->ErrLog(2)) conn->LogStream() << "User Login when not all done"<<endl;
		conn->CloseNow();
		return;
	}
	string omsg;

	/// add user to the list or fail
	if(!AddToList(conn->mpUser))
	{
		if(conn->mpUser->mClass >= eUC_REGUSER)
		{
			cUser *old_usr = FindUserNick(conn->mpUser->mNick);
			if(old_usr && old_usr->mxConn)
			{
				if(old_usr->mxConn->Log(2)) old_usr->mxConn->LogStream() << "Closing, because of his new connection" << endl;
				old_usr->mxConn->CloseNow();
				RemoveNick(old_usr->mNick);
				old_usr->mInList=false;
				if(!AddToList(conn->mpUser))
				{
					conn->CloseNow();
					return;
				}
			}
			else
			{
				conn->CloseNow();
				return;
			}
		}
		else
		{
			omsg = "You are already in the hub.";
			DCPublicHS(omsg, conn);
			conn->CloseNow();
			return;
		}
	}

	
	// prepare Hello, Myinfo, and oplist message
	omsg = "$Hello ";
	omsg+= conn->mpUser->mNick;
	if(conn->mpUser->mClass >= eUC_OPERATOR)
	{
		omsg+="|$OpList ";
		omsg+= mC.hub_security + "$$";
		GetNickListPlus1(omsg, eUC_OPERATOR,conn);
	}
	omsg+= "|$MyINFO $ALL ";
	omsg+= conn->mpUser->mNick;
	omsg+= " ";
	omsg+= conn->mpUser->mMyINFO;

	// send it to all	
	SendToAll(omsg, eUC_NORMUSER, eUC_MASTER);
	
	conn->mpUser->mInList = true;
	if(conn->Log(3)) conn->LogStream() << "Becomes in list" << endl;

	// now we can send the nicklist (if he demanded before)
	if(conn->mSendNickList)
	{
		mP.NickList(conn);
		conn->mSendNickList = false;
	}
	if(conn->Log(3)) conn->LogStream() << "Entered the hub." << endl;
	Motd(conn);

	// user has to change password
	if(conn->mRegInfo.mClass && conn->mRegInfo.mPwdChange)
	{
		omsg = "Please change your password NOW using command +passwd new_passwd!!! See +help";
		DCPrivateHS(omsg, conn);
	}

	// send the hub topic
	string topic("$HubTopic ");
	topic += mC.hub_desc;
	conn->Send(topic);
	
	conn->ClearTimeOut(eTO_LOGIN);
}

/** close a given connection with sending a message as a reason, and do it at least in to_usec microseconds */
void cServerDC::ConnCloseMsg(cConnDC *conn, const string &msg, int msec)
{
	DCPublicHS(msg,conn);
	conn->CloseNice(msec);
}

/** send the hello message, on which response we'll get MyINFO */
int cServerDC::DCHello(const string & nick, cConnDC * conn, string *info)
{
	string str("$Hello ");
	str+= nick;
	conn->Send(str);
	if(info)
		conn->Send(*info);
	return 0;
}

/** return pointer to reg info or NULL */
cRegUserInfo * cServerDC::CfgGetRegInfo(const string &nick)
{
	tRULIt it= mRegUsers.find(nick);
	if(it ==  mRegUsers.end()) return NULL;
	return it->second;
}

/** returns the nicklist i format nick$$... */
void cServerDC::GetNickListPlus1(string & list, int cls, cConnDC *conn)
{
	if(conn && conn->mpUser && !conn->mpUser->mInList && conn->mpUser->mClass >= cls)
		list+=conn->mpUser->mNick + "$$";
	GetNickList(list, cls);
}

/** return true if the condition of minimal delay is verified
and modify the timestamp */
bool cServerDC::MinDelay(time_t &then, int min)
{
	time_t now=time(NULL);
	time_t diff=now-then;
	if(min <= diff)
	{
		then = now;
		return 1;
	}
	return 0;
}

/** return true if accept is allowed - override */
bool cServerDC::AllowNewConn()
{
	return mConnList.size() <= (unsigned) mC.max_users+mC.max_extra_ops+mC.max_extra_admins + 100;
}

int cServerDC::Motd(cConnDC *conn)
{
	string mdf(".verlihub/motd");
	return SendFileHS(conn, mdf);
}

int cServerDC::SendFileHS(cConnDC *conn, const string &file)
{
	stringstream os;
	if(!conn) return 0;
	fstream is(file.data());
	if(!is.is_open()) return 0;
	string buf;
	while(!is.eof())
	{
		getline(is, buf);
		os << buf << endl;
	}
	DCPublicHS(os.str(), conn);
	is.close();
	return 1;
}

/** return the nomber of results from the sql query */
int cServerDC::ResultsCount(const string &query)
{
	if(mysql_query(&db_handle, query.data()))
	{
		cout << query << endl;
		cout << "Mysql Error: " << mysql_error( &db_handle ) << endl;
		return -1;
	}
	// take result
	MYSQL_RES *result = mysql_store_result(&db_handle);
	// count lines
	int row_count = mysql_num_rows(result);
  // user is not registered
	mysql_free_result(result);
	
	return row_count;
}

/** return 1 if user is valid and can go in
return 0 otherwise */
int cServerDC::ValidateUser(cConnDC *conn, const string &nick)
{
	// first validate the IP and host if any
	stringstream errmsg,os;
	if(!conn) return -1;
	string omsg;
	time_t n;
	bool close=false;

	// phase 1: test nick validity
	// phase 2: test ip/host ban (registered users pass)
	// phase 3: test nickban
	// then we're done
	
	// validate nick
	tVAL_NICK vn=ValidateNick(nick);
	if(vn != eVN_OK)
	{
		close=true;
		errmsg << "Bad nickname: ";
	}
	switch(vn)
	{
		case eVN_OK: break;
		case eVN_CHARS: errmsg << "unallowed characters, use these " << mC.nick_chars; break;
		case eVN_SHORT: errmsg << "too short"; break;
		case eVN_LONG: errmsg << "too long"; break;
		case eVN_USED: errmsg << "already used"; break;
		case eVN_BANNED:
			mBanList.FindBan(nick,conn->mBan);
			errmsg << "Banned." << endl;
			if( conn->mBan.mBanType )
			{
				n = conn->mBan.mTimeRest;
				errmsg << "Reason: " << conn->mBan.mReason << endl;
				if(n) errmsg
					<< "Remaining: "
					<< n/86400 << " days " << (n%86400)/3600 << " hours "
					<< (n%3600)/60 << " minutes " << n%60 << " seconds" << endl;
				else errmsg << "Permanently." << endl;
				errmsg	<< "Insert an equivalent of $5 into your floppy drive and press ENTER" << endl;
			}
			break;
		case eVN_PREFIX: errmsg << "You need to have a " << mC.nick_prefix << " prefix in fromt of your nick"; break;
		case eVN_RESERVED: errmsg <<   "reserved"; break;
		default: errmsg <<   "Uknnown error"; break;
	}

	if(close)
	{
		DCPublicHS(errmsg.str(),conn);
		return 0;
	}
	
	// validate ip
	tVAL_IP vi=ValidateIP(conn);
	// get user's reg info
	mR.FindRegInfo(conn->mRegInfo,nick);
	//=CfgGetRegInfo(nick);

	// if is not valid and user isn't registered
	if(
			vi != eVI_OK &&
			(
				!conn->mRegInfo.mClass ||
				!conn->mRegInfo.mEnabled ||
				(
					conn->mBan.mBanType &&
					conn->mBan.mNick == nick
				)
			)
		)
	{
		switch(vi)
		{
			case eVI_OK: break;
			case eVI_UNKNOWN: errmsg << "Unknwown ban"; break;
			case eVI_PRIVATE: errmsg << "Your address is from private name space, and that is not ok"; break;
			case eVI_BAN:
				errmsg << "Your ASS is permanently banned!!" <<
				endl << "Please insert an equivalent of $10 into your floppy drive and press ENTER" ;
				break;
			case eVI_T_BAN:
				n = conn->mBan.mTimeRest;
				errmsg << "You are temporarily banned, wait some time ("
							 << n/86400 << " days " << (n%86400)/3600 << " hours "
							 << (n%3600)/60 << " minutes " << n%60 << " seconds)" << endl
							 << "Insert an equivalent of $5 into your floppy drive and press ENTER" << endl;
				break;
			case eVI_BAN_RANGE: errmsg << "your network is banned"; break;
			case eVI_BAN_HOST: errmsg << "your host is banned"; break;
			default: errmsg << "unknown ip error"; break;
		}
		if(conn->mBan.mBanType)
		{
			 errmsg << "The reason is: " << conn->mBan.mReason << endl;
			 if(conn->mBan.mNick.size())
			 errmsg << "Banned nick is: " << conn->mBan.mNick;
		}
		
		// close the connection with the note
		DCPublicHS(errmsg.str().data(), conn);
		if(conn->Log(1)) conn->LogStream() << "Unallowed IP (" << vi << "), closing" << endl;
		conn->mBan.mBanType=0;
		return 0;
	}
	
	return 1;
}

/** tells if a user with such nick can login into the hub
	return
*/
tVAL_NICK cServerDC::ValidateNick(const string &nick)
{
	cUser *usr;
	if(nick.size() > mC.max_nick ) return eVN_LONG;
	if(nick.size() < mC.min_nick ) return eVN_SHORT;
	if((usr=FindUserNick(nick)))
	{
		if(usr->mxConn && usr->mClass < eUC_REGUSER)
		return eVN_USED;
	}
	if(nick.npos != nick.find_first_not_of(mC.nick_chars.data())) return eVN_CHARS;
	string query;
	query = "SELECT nick FROM nickbanlist WHERE LOWER(nick)=LOWER('";
	query+= nick;
	query+= "') AND (date_limit >= NOW() OR date_limit IS NULL)";
	if(ResultsCount(query)>0) return eVN_BANNED;
	query = "SELECT nick FROM nickreserved WHERE LOWER(nick)=LOWER('";
	query+= nick;
	query+= "')";
	if(ResultsCount(query)>0) return eVN_RESERVED;
	if(nick.compare(0,mC.nick_prefix.length(),mC.nick_prefix)!= 0) return eVN_PREFIX;
	return eVN_OK;
}

/** tells if ip can login into the hub
	return
*/
tVAL_IP cServerDC::ValidateIP(cConnDC *conn)
{
	mBanList.FindBan(conn,conn->mBan);
	if(!conn->mBan.mBanType) return eVI_OK;
	if(conn->mBan.mTimeRest == 0) return eVI_BAN;
	return eVI_T_BAN;
//	if(0 > tban)
//	{
//		if(!conn->mBan) return eVI_UNKNOWN;
//		if(conn->mBan->IsIP()) return eVI_BAN;
//		if(conn->mBan->IsHost()) return eVI_BAN_HOST;
//		if(conn->mBan->IsPriv()) return eVI_PRIVATE;
//		if(conn->mBan->IsNet()) return eVI_BAN_RANGE;
//	}
	
}

/** forces connection to move to another server, one from settings */
int cServerDC::ForceMove(cConnDC *conn)
{
	string os;
	os = "$ForceMove verliba.no-ip.com:4142";
	conn->Send(os);
	return 1;
}

/** this is called every period of time */
int cServerDC::OnTimer()
{
	if(mSendAllBuf.size())
	{
		SendToAll(mSendAllBuf, eUC_NORMUSER, eUC_MASTER);
		mSendAllBuf.clear();
	}

	// perform all temp functions
	for (tTFIt i = mTmpFunc.begin(); i != mTmpFunc.end(); i++)
	{
		if (*i)
		{
//			cout << "existing tmpfunc, timer" << endl;
//			cout << " step " << ((cInterpolExp*)(*i))->mStepsToGo << "  " << ((cInterpolExp*)(*i))->mSkipedSteps << "  "  << ((cInterpolExp*)(*i))->mSkipSteps << endl;
			// delete finished functions
			if((*i)->done())
			{
				delete *i;
				(*i) = NULL;
//				cout << " deleting tmpfunc" << endl;
			}
			else
			{
				// step the rest
				(*i)->step();
			}
		}
	}

	return 0;
}

/** converts number with units to a time period in seconds */
unsigned cServerDC::Str2Period(const string &s, ostringstream &err)
{
	static istringstream is;
	is.str(s);
	unsigned u=0;
	int m,n=0;
	char c=' ';
	is >> n >> c;
	if(n >= 0)
	{
		m = 1; // multiplicator
		if(c==' ') c = 'd';
		switch(c)
		{
			case 'y':
			case 'Y': m*= 12; // year = 12* month
			case 'M': m*=  4; // month = 4 * week
			case 'w':
			case 'W': m*=  7; // week = 7 days
			case 'd':
			case 'D': m*= 24; // 24 hours
			case 'h':
			case 'H': m*= 60; // 60mniutes
			case 'm': m*= 60; // 60 seconds
			case 's':
			case 'S': break;
			default:
				err << "Error: available units are: "
					 << "s'econd, d'ay, m'inute, h'our, d'ay, w'eek, M'onth, Y'ear." << endl
					 << "Default is 'd'." << endl;
				return 0;
				m=1;
			break;
		}
		u= n*m;
	}
	else err << "Enter a positive number." << endl;
	return u;
}

/** does the registration on given hublist */
int cServerDC::RegisterInHublist(string host, int port)
{
	ostringstream os;
	char pipe='|';
	os << pipe << mC.hub_name
		 << pipe << mC.hub_host
		 << pipe << "[SHARE:" << mC.min_share << "MB] " << mC.hub_desc
		 << pipe << mUserCount
		 << pipe << GetTotalShareSize()
		 << pipe;

	cout << os.str() << endl;
	cAsyncConn hublist(host,port);
	if(!hublist.ok) return -1;

	string lock;
	hublist.SetLineToRead(&lock,pipe,1024);
	hublist.ReadAll();
	hublist.ReadLineLocal();

	hublist.Write(os.str());
	if(!hublist.ok) cout << "Error in sending" << endl;
	hublist.close();
	return 1;
}

/** No descriptions */
long long cServerDC::GetTotalShareSize()
{
	long long total =0;
	tULIt i;
	for(i=mUserList.begin(); i!= mUserList.end(); i++)
		total += i->second->mShare;
	return total;
}

/** fill in the list of nicks with given ip */
int cServerDC::WhoIP(const string&ip, string &dest, const string&separator, bool exact)
{
	tULIt i;
	int cnt=0;
	cConnDC *conn;
	for(i=mUserList.begin(); i!= mUserList.end(); i++)
	{
		conn = (cConnDC *) i->second->mxConn;
		if(conn)
		{
			if(exact && ip == conn->AddrIP())
			{
				dest += i->second->mNick;
				dest += separator;
				cnt++;
			}
		}
	}
	return cnt;
}
};
