/***************************************************************************
                          cserverdc.h  -  description
                             -------------------
    begin                : Fri May 9 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSERVERDC_H
#define CSERVERDC_H

/**A DC server
  *@author Daniel Muller
  */

#include <mysql/mysql.h>
#include <fstream>
#include "casyncsocketserver.h"
#include "cuser.h"
#include "cmessagedc.h"
#include "creguserinfo.h"
#include "cconfigbase.h"
#include "cdcbanlist.h"
#include "cdcconsole.h"
#include "cdcconf.h"
#include "cdcproto.h"
#include "creglist.h"
#include "ctempfunctionbase.h"
#include "cmysql.h"

#if 1 // my comp
	#include <ext/hash_map>
	using namespace __gnu_cxx;
#else // dell
	#include <hash_map>
#endif

using namespace std;
using nMySQL::cMySQL;

namespace nDirectConnect
{

typedef enum
{
	eVN_OK,
	eVN_CHARS,
	eVN_SHORT,
	eVN_LONG,
	eVN_USED,
	eVN_BANNED,
	eVN_PREFIX,
	eVN_RESERVED
} tVAL_NICK;

typedef enum
{
	eVI_OK,
	eVI_UNKNOWN,
	eVI_PRIVATE,
	eVI_BAN,
	eVI_T_BAN,
	eVI_BAN_RANGE,
	eVI_BAN_HOST
} tVAL_IP;

typedef enum
{
	eMA_PROCEED,
	eMA_LIMITED,
	eMA_LATER,
	eMA_WARNING,
	eMA_IGNORE,
	eMA_HANGUP,
	eMA_HANGUP1,
	eMA_TBAN,
	eMA_ERROR

} tMsgAct;

typedef enum
{
	eSL_COOL,	// no problem
	eSL_HURRY,	// starting getting full
	eSL_SQEEZY,	// we are full, but still a little bit alive
	eSL_CRITICAL, // almost dead
	eSL_SYSTEM_DOWN // this actually never happens ;o)
} tSysLoad;

enum
{
	eULO_NONE = 0,
	eULO_GETINFO = 1
};

// forward declarations
using namespace nConfig;
using namespace nUtils;
using namespace nDirectConnect;
class cConnDC;
class cUser;


class cServerDC : public cAsyncSocketServer
{
	friend class cConnDC;
	friend class cDCConsole;
	friend class cDCProto;
	friend class cDCConf;
	friend class cRegList;
	friend class cDCBanList;
	friend class cUser;
protected:
	cDCProto mP; // protocol
	cDCConsole mCo; // console
	cDCConf mC; // configuration
	cRegList mR; // reg users
	
public:
	cServerDC(int p=0);
	virtual ~cServerDC();
	/** treat the message */
	int TreatMsg(cMessageDC *msg, cConnDC *conn);
	/** sends a public chat message to a given connection */
	int DCPublic(const string &from, const string &msg, cConnDC *conn);
	/** send a hub security public message */
	int DCPublicHS(const string &text, cConnDC *conn);
	/** tells if a user with such nick can login into the hub */
	tVAL_NICK ValidateNick(const string &nick);
	/** tells if an ip address can login into the hub */
	tVAL_IP ValidateIP(cConnDC *);

	/** return 1 if user is valid and can go in
	return 0 otherwise, and also close the connection */
	int ValidateUser(cConnDC *conn, const string &nick);

	/** try to add user into userlist, return true on success */
	bool AddToList(cUser *usr);
	/** removes user with given nick from uselist, return true on success */
	bool RemoveNick(const string &nick);
	/** find user with given nick, if not there return NULL */
	cUser * FindUserNick(const string &nick);
	/** returns the nicklist i format nick$$... */
	void GetNickList(string &list, int cls=0);
	/** send the same data to every user */
	void SendToAll(string &str, int cm,int cM);
	/** sends to every user data composed by start+nick+end */
	void SendToAllWithNick(const string &start,const string &end, int cm,int cM);
	/** The message filter, decides what to do with the message */
	tMsgAct Filter( tDCMsg msg, cConnDC * conn );
	/** return pointer to reg info or NULL if user isn't registered*/
	cRegUserInfo * CfgGetRegInfo(const string &nick);
	/** returns the nicklist i format nick$$... */
	void GetNickListPlus1(string & list, int cls, cConnDC *conn);
	/** sends by parts the nicklist the nicklist i format nick$$... */
	//void SendNickList(const char *,cConnDC *,int);
	/** send the motd */
	int Motd(cConnDC *);
	/** send all from file to a public chat of the connection */
	int SendFileHS(cConnDC *conn, const string &file);
	/** return the nomber of results from the sql query */
	int ResultsCount(const string &query);
	/** forces connection to move to another server, one from settings */
	int ForceMove(cConnDC *conn);
	/** this is called every period of time */
	int OnTimer();
	/** converts number with units to a time period in seconds */
	unsigned Str2Period(const string &, ostringstream &);
	/** send a hub security public message */
	int DCPrivateHS(const string & text, cConnDC * conn,string *from = NULL);
	/** does the registration on given hublist */
	int RegisterInHublist(string host, int port);
	/** No descriptions */
	long long GetTotalShareSize();
	/** fill in the list of nicks with given ip */
	int WhoIP(const string&ip, string &dest, const string&separator, bool exact=true);
	/** buffer to send to all */
	string mSendAllBuf;

public: // Public attributes
	/** system load indicator */
	tSysLoad mSysLoad;
	/** mysql database handle */
	MYSQL db_handle;
	// mysql database connection
	cMySQL mMySQL;
	/** last op that user broadcast function */
	string LastBCNick;
	/** oplist prefabricated */
	string mOpListString;
	/** indicates if nicklist is to be remade */
	bool mRemakeNickList;
	/** network output log */
	ofstream mNetOutLog;

	/** nick comparisson structure */
	struct eEqNick
	{
		bool operator()(const string &s1, const string &s2) const{	return s1 == s2;}
	};
	/** hashing structure */
	struct hHashStr
	{
		static hash<const char *> msHash;
		size_t operator() (const string &s) const{return msHash(s.data());}
	};
	/********** typedefs */
	/** the userlist type */
	typedef class hash_map<string, cUser *, hHashStr, eEqNick> tUsrList;
	typedef pair<string,cUser*> tNickUserPair;
	typedef tUsrList::iterator tULIt;
	/** trigger functions type */
	typedef int tDC_MsgFunc(cMessageDC * msg, cConnDC * conn);
	typedef class hash_map<string, cRegUserInfo *, hHashStr, eEqNick> tRegUsrList;
	typedef pair<string,cRegUserInfo*> tRegUsrPair;
	typedef tRegUsrList::iterator tRULIt;
	/** temporary time functions */
	typedef vector<cTempFunctionBase *> tTmpFunc;
	typedef tTmpFunc::iterator tTFIt;

	/** list of temporary functions */
	tTmpFunc mTmpFunc;

private: //Private attributes
	/** the userlist */
	tUsrList mUserList;
	/** reg user's list list */
	tRegUsrList mRegUsers;

protected: // Protected methods
	/** create a connection object to the sock descriptor */
	cAsyncConn *FactoryConnection(int sd);
	/** create somehow a string to get line for given connection, ad return th pointer */
	string * FactoryString(cAsyncConn * );
	/** treat message for given connection */
	void OnNewMessage(cAsyncConn * , string * );
	/** loads te list of registered users */
	int CfgLoadReglist();
	/** is called when user is about to get into the userslist */
	void DoUserLogin(cConnDC *conn);
	/** close a given connection with sending a message as a reason, and do it at least in to_usec microseconds */
	void ConnCloseMsg(cConnDC *conn, const string &msg, int to_msec=4000);
	/** send the hello messahe, on which response we'll get MyINFO */
	int DCHello(const string & nick, cConnDC * conn, string *info=NULL);
	/** return true if the condition of minimal delay is verified
	    and modify the timestamp */
	bool MinDelay(time_t &what, int min);
	/** return true if accept is allowed - override */
	bool AllowNewConn();
	/** return negative if conn should be removed */
	int OnNewConn(cAsyncConn * );

protected: // Protected attributes
	/** the number of allowed users */
	int mUserCount;
	long mTotalShare;
	/** banlist */
	cDCBanList mBanList;
	/** when hub started */
	time_t mStartTime;
	/** string containing nicks separated by $$ */
	string mNickListString;
};

};
#endif
