/***************************************************************************
                          cuser.cpp  -  description
                             -------------------
    begin                : Thu May 8 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cuser.h"

namespace nDirectConnect
{

cTime user_global_time;

cUser::cUser() :
	cObj((const char *)"User"),
	mShare(0),
	mFloodPM(0.,30.,10,user_global_time.Get())
{
	mxConn = NULL;
	mxServer = NULL;
	mGag = false;
	mProtectFrom = 0;
	mHideKick = false;	
}

/** Constructor */
cUser::cUser(const string &nick) :
	mNick( nick ),
	mxConn(NULL),
	mxServer(NULL),
	mShare(0),
	mHideKicksForClass(eUC_NORMUSER),
	mFloodPM(0.5,30.,30,user_global_time.Get())
{
	mInList = false;
	mClass = eUC_NORMUSER;
	mRights = 0;
	mToBan = false;
	mVisibleClassMin = eUC_NORMUSER;
	mOpClassMin = eUC_NORMUSER;
	mGag = false;
	mProtectFrom = 0;
	mHideKick = false;
	#warning HideKick is to be replaced by usage HideKickForCLass
}

cUser::~cUser()
{
	if(mInList && mxServer) mxServer->RemoveNick(mNick);
}


/** return tru if user needs a password and the password is correct */
bool cUser::CheckPwd(const string &pwd)
{
	if(!mxConn || !mxConn->mRegInfo.mClass) return false;
	return mxConn->mRegInfo.PWVerify(pwd);
}

/** perform a registration: set class, rights etc... precondition: password was al right */
void cUser::Register()
{
	if(!mxConn || !mxConn->mRegInfo.mClass) return;
	mClass = (tUserCl)mxConn->mRegInfo.mClass;
}

};

/*!
    \fn nDirectConnect::cUser::ShareEnthropy()
 */
long nDirectConnect::cUser::ShareEnthropy(const string &sharesize)
{
	char diff[20];
	int count[20];

	int i,j;
	long score = 0;
	// calculate counts of every byte in the sharesize string
	for( i = 0; i < sharesize.size(); i++ )
	{
		count[i] = 0;
		for( j = i+1; j < sharesize.size(); j++ )
			if(sharesize[i]==sharesize[j]) ++ count[i];
	}

	// make the weighted sum
	for( i = 0; i < sharesize.size(); ) score += count[i]*++i;

	// calculate the differences
	for( i = 0; i < sharesize.size()-1; i++ )
		diff[i] = 10 + sharesize[i-1] - sharesize[i];

	// calculate counts of every byte in the sharesize string differences
	for( i = 0; i < sharesize.size()-1; i++ )
	{
		count[i] = 0;
		for( j = i+1; j < sharesize.size(); j++ )
			if(diff[i]==diff[j]) ++ count[i];
	}
	for( i = 0; i < sharesize.size(); ) score += count[i]*++i;

	return score;
}

