/***************************************************************************
                          cuser.h  -  description
                             -------------------
    begin                : Thu May 8 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CUSER_H
#define CUSER_H

#include <string>
#include "cobj.h"
#include "cconndc.h"
#include "cserverdc.h"
#include "cfreqlimiter.h"
#include <time.h>

using namespace std;

namespace nDirectConnect
{

/**Any type of dc user, contains info abou the connected users
  *@author Daniel Muller
  */

/** several types of users with some differences .. for later
	everyone is allowed to create no more then he is -1
*/
typedef enum
{
	eUC_NORMUSER = 0,
	eUC_REGUSER = 1,
	eUC_VIPUSER = 2,
	eUC_OPERATOR = 3,
	eUC_CHEEF = 4,
	eUC_ADMIN = 5,
	eUC_MASTER = 10
} tUserCl;


/** user rights, there will be defaults for every class, but they can be changed */
typedef enum
{
	eUR_NOINFO  = 0x000001, // can login without user info
	eUR_NOSHARE = 0x000002, // can login with less than share limit
	eUR_CHAT		= 0x000004, //  can talk in the main chat
	eUR_SEARCH  = 0x000008, // can search
	eUR_STPM    = 0x000010, // stealth PM (with other nick, that doesn't exist or not registered)
	eUR_STCHAT	= 0x000020, // can stealth chat (with changing his nick to a nonexistent other nick)
	eUR_REDIR   = 0x000040, // can op force move to a selected hublist
	eUR_REDIRANY= 0x000080, // can op force move to any hub
	eUR_KICK    = 0x000100, // can kick (with a previous chat message)
	eUR_DROP    = 0x000200, // can drop users (without the chat message)
	eUR_BANBIT1	= 0x000400, // 
	eUR_BANBIT2	= 0x000800, // four levels of (un)ban (NICK,D,W,PERM) bit1+2*bit2	
	eUR_GETIP  	= 0x001000, // get user's ip
	eUR_FULL1   = 0x002000, // connection on almost full hub
	eUR_FULL2   = 0x004000, // connection on completely full hub (someone is doconnected)
	eUR_MASSMSG = 0x008000,
	eUR_MASSRED = 0x010000, // masss redirect
	eUR_S_MAXU  = 0x020000, // set max users
	eUR_S_MINS  = 0x040000, // set minshare
	eUR_S_HUBN  = 0x080000, // set hubname
	eUR_S_REDI	= 0x100000  // set redirhub(s) etc...
} tUserRights;

/** I should define for each class of users a mask of rights that they can't get ad of
 * those that they always get.. this should be configurable 2DO TODO*/

class cConnDC;
class cServerDC;

class cUser : public cObj
{
public: 
	cUser();
	~cUser();
	/** constructor with a nickname */
	cUser(const string &nick);
	/** check for the right to ... */
	inline int HaveRightTo(unsigned int mask){ return mRights & mask; }
	/** return tru if user needs a password and the password is correct */
	bool CheckPwd(const string &pwd);
	/** perform a registration: set class, rights etc... precondition: password was al right */
	void Register();
public: // Public attributes
	/** user's  nickname */
	string mNick;
	/** pointer to the connection */
	cConnDC * mxConn;
	/** pointer to the srever
		this is extern pointer do not delete */
	cServerDC * mxServer;
	/** if the user was already added to the list */
	bool mInList;
	/** the part of MyINFO command, that is commun for all */
	string mMyINFO;
	/** the uers's class */
	tUserCl mClass;
	/** user's rights */
	unsigned long mRights;
	/** different time stamps */
	struct sTimes
	{
		/******** [s] ****/
		/** connection time */
		time_t connect;
		/** login time, user entered userlist */
		time_t login;
		/** last search */
		time_t search;
		/** my_info */
		time_t info;
		/** chat */
		time_t chat;
		/** private message to any user*/
		time_t pm;
		sTimes(){connect=login=info=chat=pm=search=0;}
	};
	sTimes mT;
	/** 0 means perm ban, otherwiese in seconds */
	time_t mBanTime;
	/** indicates whether user is to ban after the following kick */
	bool mToBan;
	/** ban reason */
	string mBanReason;
	/** minimal class users that can see this one */
	tUserCl mVisibleClassMin;
	/** minimal class users that can see this one as operator */
	tUserCl mOpClassMin;
	/** user's sharesize */
	long long mShare;
	/** the list of nicks queued to send either as to nicklist or myinfo. */
	string mQueueUL;
	/** ip, that user has for downloads ises in connect to me */
	string mIP4DL;
	/** specifies the one bove higher class of users can see kick chat mesages */
	int mHideKicksForClass;
	/** flood detectors */
	cFreqLimiter mFloodPM;
	/** chat discrimionation */
	bool mGag;
	/** kick messages hide from chat */
	bool mHideKick;
	/** class protection against kicking */
	int mProtectFrom;

protected: // Protected attributes
	// $MyINFO $ALL <nick> <interest>$ $<speed>$<e-mail>$<sharesize>$
	/** the dc++ or dcgui tags */
	string mTag;
	/** the interrest (description without tag) */
	string mInterest;
	/** the rest of myinfo */
	string mInfo;


	public: long ShareEnthropy(const string &sharesize);
};
};

#endif
